/* Copyright (c) 2009, Sean A.O. Harney */

import java.awt.event.*;

import java.io.*;

import javax.swing.*;

public class MainMenuBar extends JMenuBar implements ActionListener {
	JFileChooser fileChooser, dirChooser;
	LookAndFeel origLaF = UIManager.getLookAndFeel();

	public MainMenuBar() {
		super(); // JMenuBar(), needed?
		this.add(buildFileMenu());
		this.add(buildHelpMenu());
		this.toggleMotifLookAndFeel(true); // create file choosers with Motif
		// look and feel (easier to use)
		fileChooser = new JFileChooser(); // should recreate each time?
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY); // only allow
		// files to
		// be
		// choosen
		dirChooser = new JFileChooser();
		dirChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		this.toggleMotifLookAndFeel(false);
	}

	// for the JMenuItems
	public void actionPerformed(ActionEvent e) {
		String source = ((JMenuItem) (e.getSource())).getName();
		Jkmt frame = Jkmt.getInstance(); // get singleton instance

		if (source.equals("file.opendir")) {

			// toggleMotifLookAndFeel(true); //Already were instantiated with
			// the motif LaF.
			int res = dirChooser.showOpenDialog(frame);

			// toggleMotifLookAndFeel(false);
			if (res == JFileChooser.APPROVE_OPTION) {

				// close all tabs? Perhaps leave them open since each tab is a
				// thread which tail -fs the file, no need to close. TODO: make
				// tail -f option
				// set tabs
			}
		} else if (source.equals("file.openfile")) {
			int res = fileChooser.showOpenDialog(frame);

			if (res == JFileChooser.APPROVE_OPTION) {
				File f = fileChooser.getSelectedFile();

				// add a single tab for it
				// check it is not open already, if it is popup warning,
				// try {
				// JPanel newTab = new IStreamScrollPanel(new DataInputStream(
				// new FileInputStream(f)), false);
				JPanel newTab = new JPanel();
				frame.tabPane.add(f.getName(), newTab);
				// } catch (FileNotFoundException e1) {
				// e1.printStackTrace();

				// shouldnt happen unless file was removed between choosing
				// and now.
				// return;
				// }
			}
		} else if (source.equals("file.exit")) {
			frame.exit();

			// Doesn't return
		} else if (source.equals("help.about")) {
			ImageIcon icon = new ImageIcon("images/icon.gif", "Jkmt logo icon");
			String COPYRIGHT = "\u00a9";

			JOptionPane.showMessageDialog(frame, COPYRIGHT
					+ " 2009 Innovative Security Solutions, Inc.",
					"About Innosecc Jkmt", JOptionPane.INFORMATION_MESSAGE,
					icon);
		}
	}

	private JMenu buildFileMenu() {
		JMenu fileMenu = new JMenu("File");

		fileMenu.setMnemonic(KeyEvent.VK_F);

		JMenuItem mi = new JMenuItem("Open Directory", new ImageIcon(
				"images/midle.gif"));

		mi.setName("file.opendir");
		mi.setMnemonic(KeyEvent.VK_O);
		mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D,
				ActionEvent.CTRL_MASK));
		mi.addActionListener(this);
		fileMenu.add(mi);
		mi = new JMenuItem("Open File", new ImageIcon("images/midle.gif"));
		mi.setName("file.openfile");
		mi.setMnemonic(KeyEvent.VK_T);
		mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F,
				ActionEvent.CTRL_MASK));
		mi.addActionListener(this);
		fileMenu.add(mi);
		fileMenu.addSeparator();
		mi = new JMenuItem("Exit", new ImageIcon("images/midle.gif"));
		mi.setName("file.exit");
		mi.setMnemonic(KeyEvent.VK_X);
		mi.addActionListener(this);
		fileMenu.add(mi);

		return fileMenu;
	}

	private JMenu buildHelpMenu() {
		JMenu helpMenu = new JMenu("Help");

		helpMenu.setMnemonic(KeyEvent.VK_H);

		JMenuItem mi = new JMenuItem("lol", new ImageIcon("images/midle.gif"));

		mi.setName("help.lol");
		mi.setMnemonic(KeyEvent.VK_O);
		mi.addActionListener(this);
		helpMenu.add(mi);
		mi = new JMenuItem("About Jkmnt", new ImageIcon("images/midle.gif"));
		mi.setName("help.about");
		mi.setMnemonic(KeyEvent.VK_A);
		;
		mi.addActionListener(this);
		helpMenu.add(mi);

		return helpMenu;
	}

	// want to use only he motif look and feel for the file chooser
	private void toggleMotifLookAndFeel(boolean on) {
		try {
			String laf = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";

			if (on) {
				UIManager.setLookAndFeel(laf);
			} else {
				UIManager.setLookAndFeel(origLaF);
			}
		} catch (Exception e1) {

			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	}
}
