//~--- JDK imports ------------------------------------------------------------

import java.awt.*;
import java.awt.event.*;

import java.io.*;

import java.util.ArrayList;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileSystemView;

class ProfileRecord implements Serializable {
	String hostName; // hostname or IP address
	String name;
	String passwd; // later store with a reversible encryption
	int port;
	boolean ssl;

	// passwd,cert string etc.
	//  
	public ProfileRecord(String name, String hostName, int port, boolean ssl,
			String passwd) {
		this.name = new String(name);
		this.hostName = new String(hostName); // necessary to wrap ?
		this.port = port;
		this.ssl = ssl;
		this.passwd = passwd;
	}
}

public class ServerTab extends JPanel implements ActionListener,
		ListSelectionListener {
	JButton cbutton = new JButton("Connect");
	JButton delButton = new JButton("Delete Selected");
	JTextField hostField = new JTextField(25);
	Jkmt jkmt = Jkmt.getInstance();
	JTextField nameField = new JTextField(10);
	JPasswordField passwdField = new JPasswordField(15);

	JTextField portField = new JTextField(5);
	JList profileList = null;
	ArrayList<ProfileRecord> profileRecords = new ArrayList<ProfileRecord>();
	File profilesFile = null;
	JButton saveButton = new JButton("Save");
	JCheckBox sslCheckBox = new JCheckBox("SSL", false);

	public ServerTab() {
		super();
		setLayout(new BorderLayout());

		FileSystemView fsv = FileSystemView.getFileSystemView();
		File homeDir = fsv.getHomeDirectory();

		System.out.println("homeDir: " + homeDir);

		String filePath = homeDir.getPath() + File.separator + ".JkmtProfiles";

		System.out.println("filePath: " + filePath);
		profilesFile = new File(filePath);

		if (profilesFile.isFile()) // exists and is a file
		{
			jkmt.consolePanels.printlnJkmt("Found profiles file: reading");
			readProfilesFile();
		} else {
			jkmt.consolePanels
					.printlnJkmt("The profiles file does not exist: creating");
			createProfilesFile();
		}

		JPanel form = new JPanel();
		form.setLayout(new FlowLayout(FlowLayout.LEADING));
		form.add(new JLabel("Innosecc KMT Server hostname or IP address: "));
		form.add(hostField);
		form.add(new JLabel("port: "));
		form.add(portField);

		JPanel form1 = new JPanel();
		form1.setLayout(new FlowLayout());
		form1.add(new JLabel("Password: "));
		form1.add(passwdField);
		form1.add(sslCheckBox);
		cbutton.setActionCommand("connect");
		cbutton.setMnemonic(KeyEvent.VK_C);
		cbutton.addActionListener(this);
		form1.add(cbutton);

		JPanel topform = new JPanel();
		topform.setLayout(new BoxLayout(topform, BoxLayout.Y_AXIS));
		topform.add(new JSeparator());
		topform.add(form);
		topform.add(form1);
		topform.add(new JSeparator());

		add(topform, BorderLayout.NORTH);

		JPanel profilesForm = new JPanel();

		profilesForm.setLayout(new FlowLayout());

		JPanel p = new JPanel();

		p.setLayout(new BorderLayout());

		JPanel pn = new JPanel();

		pn.setLayout(new BoxLayout(pn, BoxLayout.Y_AXIS));
		pn.add(new JLabel("Choose, Save, or Delete a Stored Jkmt Profile"));
		pn.add(new JSeparator());
		pn.add(new JSeparator());
		pn.add(new JSeparator());
		pn.add(nameField);
		pn.add(new JSeparator());
		pn.add(new JSeparator());// appears to be 3 since top of component under
		// it is a line
		p.add(pn, BorderLayout.NORTH);

		// String[] listStrings = { "default" };
		String[] listStrings = new String[profileRecords.size()];
		int i = 0;

		for (ProfileRecord pr : profileRecords) {
			listStrings[i++] = pr.name;
		}

		profileList = new JList(listStrings);
		profileList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		profileList.setSelectedIndex(0); // the default is first in
		// file/arraylist
		setSwingFields(profileRecords.get(0)); // set fields to default

		ListSelectionModel lsm = profileList.getSelectionModel();

		lsm.addListSelectionListener(this);

		JScrollPane scrollPane = new JScrollPane(profileList,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);

		// NOTE: x dimension is being stretched larger to JTextfield size if tht
		// is larger
		scrollPane.setPreferredSize(new Dimension(200, 100));
		p.add(scrollPane, BorderLayout.CENTER);
		profilesForm.add(p);

		JPanel east = new JPanel();

		east.setLayout(new BoxLayout(east, BoxLayout.Y_AXIS));
		saveButton.setActionCommand("save");
		Dimension size = new Dimension(130, (int) saveButton.getPreferredSize()
				.getHeight());
		saveButton.setPreferredSize(size);
		saveButton.setMinimumSize(size);
		saveButton.setMaximumSize(size);
		saveButton.setMnemonic(KeyEvent.VK_S);
		saveButton.addActionListener(this);
		east.add(saveButton);

		delButton.setActionCommand("delete");
		delButton.setPreferredSize(size);
		delButton.setMinimumSize(size);
		delButton.setMaximumSize(size);
		delButton.setMnemonic(KeyEvent.VK_D);
		delButton.setPreferredSize(new Dimension(130, delButton
				.getPreferredSize().height));
		delButton.setEnabled((lsm.getMinSelectionIndex() == 0) ? false : true);
		delButton.addActionListener(this);
		east.add(delButton);
		profilesForm.add(east);
		add(profilesForm, BorderLayout.CENTER);
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("connect")) {
			cbutton.setText("Disconnect");
			cbutton.setActionCommand("disconnect"); // switch action command,
			// but don't enable editable
			// things until after actual
			// connect in Net.c
			hostField.setEnabled(false);
			portField.setEnabled(false);
			passwdField.setEnabled(false);
			sslCheckBox.setEnabled(false);
			jkmt.consolePanels.printlnJkmt("Connecting to " + hostField.getText()
					+ ":" + portField.getText() + "\n");
			jkmt.net = new Net();
			jkmt.net.execute();
		} else if (e.getActionCommand().equals("disconnect")) {

			try {
				if (jkmt.net != null && jkmt.net.socket != null)
					jkmt.net.socket.close();
			} catch (IOException ioe) {
				ioe.printStackTrace();

				// do soemthig else.
			}
			jkmt.net.cancel(true); // shut down this thread, allow interruption.
//NOTE: important this is here, if network socket disconnect, does and should not clear everything out. onlyclear out everythingfor this explicit disconnectGUI event 

	if(Net.paneAlert != null)
	{
	Net.paneAlert.setVisible(false); // TODO: would rather destroy them but minimizing is fine for now, since new ones are created
	Net.paneAlert = null;
	Net.countAlert=0;
		}
	if(Net.paneTf != null)
	{
	Net.paneTf.setVisible(false);
    Net.paneTf = null;
     	Net.countTf = 0;
	}
				
			jkmt.xmlEditorPanel.xTree.refresh("<empty></empty>");
			jkmt.xmlEditorPanel.xTree.setEnabled(false);

			/*
			 * every tab but server which does it manually and doesnt implement
			 * method.
			 */
			jkmt.optionsTab.disableAll();
			jkmt.dpsTab.disableAll();
			jkmt.kmemstatTab.disableAll();
			jkmt.ksymstatTab.disableAll();
			jkmt.ktraqTab.disableAll();
			jkmt.drchkTab.disableAll();

			/* remove hourglass cursor, just incase it is still set */
			jkmt.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));

			cbutton.setText("Connect");
			cbutton.setActionCommand("connect"); // switch action command
		} else if (e.getActionCommand().equals("save")) { // saves whole
			// arraylist to file
			String name = nameField.getText();
			String hostf = hostField.getText();
			String passwdf = passwdField.getText();
			int portf = Integer.parseInt(portField.getText());

			// or host or port.. TODO: check before parseInt
			if ("".equals(name)) { /* error */
			}

			boolean foundExisting = false;

			for (ProfileRecord pr : profileRecords) {
				if (pr.name.equals(name)) {
					jkmt.consolePanels.printlnJkmt("Updating existing Profile: "
							+ name);
					pr.hostName = hostf;
					pr.port = portf;
					pr.ssl = sslCheckBox.isSelected();
					pr.passwd = passwdf;
					foundExisting = true;

					break;
				}
			}

			if (!foundExisting) {
				jkmt.consolePanels.printlnJkmt("Creating new Profile: " + name);

				ProfileRecord pr = new ProfileRecord(name, hostf, portf,
						sslCheckBox.isSelected(), passwdf);

				profileRecords.add(pr); // add to ArrayList

				String[] listStrings = new String[profileRecords.size()];
				int i = 0;

				for (ProfileRecord pr1 : profileRecords) {
					listStrings[i++] = pr1.name;
				}

				profileList.setListData(listStrings);
				profileList.setSelectedIndex(i - 1); // the new one added is
				// selected
			}

			jkmt.consolePanels.printlnJkmt("Serializing to " + profilesFile);
			writeProfilesFile();
		} else if (e.getActionCommand().equals("delete")) {

			/* get pr for selected, not name from textArea */
			ListSelectionModel lsm = profileList.getSelectionModel();
			int index = lsm.getMinSelectionIndex();

			if (index == -1) {
				return;
			}

			// TODO: popup confirm are you sure?
			ProfileRecord pr = profileRecords.remove(index);
			String[] listStrings = new String[profileRecords.size()];
			int i = 0;

			for (ProfileRecord pr1 : profileRecords) {
				listStrings[i++] = pr1.name;
			}

			profileList.setListData(listStrings);
			profileList.setSelectedIndex(0); // after deleting, default is
			// selected.
		}
	}

	// NOTE: only call if you checked !isFile
	private void createProfilesFile() {
		try {
			profilesFile.createNewFile();

			ProfileRecord pr = new ProfileRecord("default", "localhost", 999,
					true, "");

			profileRecords.add(pr);
			writeProfilesFile();
		} catch (java.io.IOException ioe) {
			ioe.printStackTrace();
		}
	}

	private void readProfilesFile() {
		try {
			ObjectInputStream inputStream = new ObjectInputStream(
					new FileInputStream(profilesFile));

			profileRecords = (ArrayList<ProfileRecord>) inputStream
					.readObject();
			inputStream.close();
		} catch (java.io.IOException ioe) {
			ioe.printStackTrace();
		} catch (ClassNotFoundException cnfe) {

			// popup error??
			cnfe.printStackTrace();
		}

		jkmt.consolePanels
				.printlnJkmt("Read " + profileRecords.size() + " profiles");
	}

	private void setSwingFields(ProfileRecord pr) {
		nameField.setText(pr.name);
		hostField.setText(pr.hostName);
		portField.setText("" + pr.port);
		sslCheckBox.setSelected(pr.ssl);
		passwdField.setText(pr.passwd);
	}

	public void valueChanged(ListSelectionEvent e) {
		if (e.getValueIsAdjusting()) { // wait for last one
			return;
		}

		ListSelectionModel lsm = (ListSelectionModel) e.getSource();
		int index = lsm.getMinSelectionIndex();

		if (index == -1) {
			return;
		}

		// System.out.println(""+index);
		//      

		/* disable delbutton for default entry */
		delButton.setEnabled((index == 0) ? false : true);

		ProfileRecord pr = profileRecords.get(index);

		setSwingFields(pr);
	}

	private void writeProfilesFile() {
		try {
			ObjectOutputStream outputStream = new ObjectOutputStream(
					new FileOutputStream(profilesFile));

			outputStream.writeObject(profileRecords); // serialize arrayList
			outputStream.flush();
			outputStream.close();
		} catch (java.io.IOException ioe) {
			ioe.printStackTrace();
		}

		jkmt.consolePanels.printlnJkmt("Wrote " + profileRecords.size()
				+ " profiles");
	}
}
