/* 
 * Functions to read kernel memory snapshots of text segment
 * Author: Ryan O'Neill <ryan@innosecc.com>
 * (C) Innovative Security Concepts 2009
 *
 */


/* i.e fd = open_snapshot("snap.img", O_RDONLY); */

/* We use global snap_fd to maintain compatibility with readkmem() function */

/* To easily convert existing tools that use /dev/kmem to have snapshot functionality */
/* We give our open/read snapshot functions the same prototype as the read kmem ones */
/* and create function ptrs in tools such as dps, ksymstat, kmemstat, ktraq like this: */
/*
        int (*openkmem_ptr)(char *, int);
        int (*readkmem_ptr)(unsigned char *, unsigned, int);
        void (*closekmem_ptr)(void);
	
	(DEFAULT PTR ASSIGNMENT)
        openkmem_ptr = openkmem;
        readkmem_ptr = readkmem;
        closekmem_ptr = closekmem;

	(IF Snapshot option is used then)
	openkmem_ptr = open_snapshot;
	openkmem_ptr = read_snapshot;
	closekmem_ptr = close_snapshot;

	And we modify our existing tools to obviously use openkmem_ptr instead fo openkmem 
	It's a fairly transparent addition using this method. I've currently added the feature
	to DPS already. i.e ./dps /System.map -S kmem.snapshot (Or whatever the path is)
*/
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <stdio.h>

int snap_fd;

#define SNAPSHOT_TEXT_BASE 0xc0100000

int open_snapshot(char *path, int perms)
{

	if ((snap_fd = open(path, perms)) < 0)
	{
		printf("[TOOL_FAILURE] in open_snapshot() unable to open %s - %s\n", path, strerror(errno));
		return -1;
	}
	
	return 0;
		
}

int read_snapshot(unsigned char *mem, unsigned off, int bytes)
{
	int c;
		
	unsigned int offset = off - SNAPSHOT_TEXT_BASE;
	
	printf("vaddr: %x calculates to %x\n", off, offset);

	if (lseek(snap_fd, offset, SEEK_SET) != offset)
	{
	        printf("[TOOL_FAILURE] read_snapshot-> (lseek) %s\n", strerror(errno));
                return -1;
        }

	if ((c = read(snap_fd, (unsigned char *)mem, bytes)) != bytes)
	{
		printf("[TOOL_FAILURE] read_snapshot-> (read %d bytes)\n", c);
		return -1;
	}
}

void close_snapshot(void)
{
	close(snap_fd);
}

		

	


	
	
	

