#include "common.h"



SSL *ssl_accept (int cipher_sock, SSL_CTX *ctx)
{
    
	BIO *sbio = BIO_new_socket (cipher_sock, BIO_NOCLOSE);
	SSL *ssl = SSL_new (ctx);
	int r;

	if(ssl == NULL)
	{
	    	fprintf(stderr, "SSL_new(): ");
	    	ERR_print_errors_fp(stderr);
		return NULL;
	}

	SSL_set_bio (ssl, sbio, sbio);
	

	if ((r = SSL_accept (ssl)) <= 0)
	{
	    	fprintf(stderr, "SSL_accept(): SSL handshake failure: %s\n", 			    ERR_error_string(SSL_get_error(ssl, r), NULL));
		ssl_close(ssl);
		return NULL;
	}

	return ssl;
}

int ssl_send (SSL * ssl, void *buf, size_t count)
{
	int r = SSL_write (ssl, (char *) buf, count);
	if (r <= 0)
	{
		//openssl_print_error (ssl, r, 1);
		return -1;
	}
	return r;

}

int ssl_recv (SSL * ssl, void *buf, size_t count)
{
	int r = SSL_read (ssl, (char *) buf, count);
	if (r <= 0)
	{
		//openssl_print_error (ssl, r, 1);
		return -1;
	}
	return r;
}


int ssl_close (SSL * ssl)
{
	SSL_shutdown (ssl);
	SSL_free (ssl);
	return 0;
}

//TODO: return NULL on failure
SSL_CTX *ssl_init (const char *server_cert_pem, const char *server_privkey_pem)
{
	SSL_CTX *ctx;
	SSL_library_init ();
	SSL_load_error_strings ();

	SSL_METHOD *meth;
	
	meth = SSLv23_method ();
	ctx = SSL_CTX_new (meth);

 	if (!SSL_CTX_use_PrivateKey_file(ctx, server_privkey_pem, SSL_FILETYPE_PEM))
 	{
		printf ("Can't read private key file %s\n", server_privkey_pem);
		return NULL;
 	}

	if (!(SSL_CTX_use_certificate_file (ctx, server_cert_pem, SSL_FILETYPE_PEM)))
	{
		printf ("Can't read certificate file %s\n", server_cert_pem);
		return NULL;
	}
	SSL_CTX_set_verify(ctx, SSL_VERIFY_NONE, NULL);
	return ctx;
}
