/*
 * dr_chk v0.1 (C) innosecc.com
 * author: Ryan O'Neill
 * This software checks the INT 1 handler to see if it
 * has a patched call to do_debug, which should detect
 * Debug Register rootkits
 */

/* Here is the aspect of the DR rootkit (Immunities example) */
/* That we are taking advantage of in our detection method */
/*
static int __get_and_set_do_debug_2_6(unsigned int handler, unsigned int my_do_debug)
{
    unsigned char *p        = (unsigned char *)handler;
    unsigned char buf[4]    = "\x00\x00\x00\x00";
    unsigned int offset     = 0;
    unsigned int orig       = 0;

    while (p[0] != 0xe8)
    {
        p ++;
    }
    DEBUGLOG(("*** found call do_debug %X\n", (unsigned int)p));
    buf[0]  = p[1];
    buf[1]  = p[2];
    buf[2]  = p[3];
    buf[3]  = p[4];

    offset  = *(unsigned int *)buf;
    DEBUGLOG(("*** found call do_debug offset %X\n", offset));

    orig    = offset + (unsigned int)p + 5;
    DEBUGLOG(("*** original do_debug %X\n", orig));

    offset  = my_do_debug - (unsigned int)p - 5;
    DEBUGLOG(("*** want call do_debug offset %X\n", offset));

    p[1]    = (offset & 0x000000ff);
    p[2]    = (offset & 0x0000ff00) >>  8;
    p[3]    = (offset & 0x00ff0000) >> 16;
    p[4]    = (offset & 0xff000000) >> 24;
    DEBUGLOG(("*** patched in new do_debug offset\n"));

    return orig;
}
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <strings.h>
#include <string.h>
#include <sys/mman.h>
#include <getopt.h>
#include <errno.h>

/* Types to pass get_symbol */
#define KALLSYMS 0
#define SYSMAP 1

/* TODO this kmem_off_t should be moved into a global header */
/* as I redefine this in several tools */

#ifdef x86_64
typedef unsigned long kmem_off_t;
#else
typedef unsigned long long kmem_off_t;
#endif

int kfd;

/* TODO both readkmem and get_symbol should eventually be put into common */
int readkmem (unsigned char *mem, unsigned long off, int bytes)
{
        int c;
	
        if (lseek64 (kfd, (kmem_off_t) off, SEEK_SET) != off)
        {
                printf("[TOOL_FAILURE] (lseek64) %s\n", strerror(errno));
                return -1;
        }

        if ((c = read (kfd, mem, bytes)) != bytes)
        {
                printf("[TOOL_FAILURE] (read %d bytes) %s\n", c, strerror(errno));
                return -1;
        }
	
	return 0;
}

unsigned long get_symbol(char *name, char type, char *path)
{
        FILE *fd;
        char symbol_s[255];
        int i;
        unsigned long address, next_symaddr;
        char tmp[255], tmp_type;

        if ((fd = fopen(path, "r")) == NULL)
        {
		printf("[TOOL_FAILURE] fopen %s %s\n", path, strerror(errno));
                exit(-1);
        }

        while(!feof(fd))
        {
                bzero((char *)symbol_s, 255);
                fscanf(fd, "%x %c %s", &address, &type, &symbol_s);

                if (strcmp(symbol_s, name) == 0)
                {
                        fscanf(fd, "%x %c %s", &next_symaddr, &tmp_type, &tmp);
                        fclose(fd);
                        return address;
                }
                /* kallsyms isn't a regular file, meaning there is no EOF */
                if (!strcmp(symbol_s, ""))
                        break;
        }
        return 0;
}

int main(int argc, char **argv)
{
	int c, i;
	int smap = 0, kmt_mem = 0;
	int file_type = KALLSYMS;
	extern char *optarg;
	char *sympath, *kpath;
	unsigned char code[64], *p;
	long do_debug_off;
	unsigned long do_debug_vaddr;
	unsigned long real_do_debug_vaddr;

	/* debug_stack_correct address */
	unsigned long dsc_vaddr;
	
	/* Our only option is to use System.map over kallsyms */
	while ((c = getopt(argc, argv, "hMS:")) != -1)
	{
		switch (c)
		{
			case 'S':
				smap++;
				file_type = SYSMAP;
				if ((sympath = strdup(optarg)) == NULL)
				{
					printf("[TOOL_FAILURE] strdup %s\n", strerror(errno));
					exit(-1);
				}
				break;
			case 'M':
				kmt_mem++;
				if ((kpath = strdup("/dev/kmt_mem")) == NULL)
				{
					printf("[TOOL_FAILURE] strdup %s\n", strerror(errno));
                                        exit(-1);
                                }
				break;
			case 'h':
			default:
				printf("Usage: %s [-M] [-S <sysmap>[-h (Help)]\n"
				       "[-S <System.map path>]\tUse system.map instead of default /proc/kallsyms.\n"
				       "[-M]	(-M option will use /dev/kmt_mem instead of /dev/kmem)\n", argv[0]);
				exit(c == 'h' ? 0 : -1);
				break; 
		}
	}

	/* We use /proc/kallsyms by default */
	if (!smap)
		if ((sympath = strdup("/proc/kallsyms")) == NULL)
                {
                	printf("[TOOL_FAILURE] strdup %s\n", strerror(errno));
                        exit(-1);
                }
	
	if (!kmt_mem)
		if ((kpath = strdup("/dev/kmem")) == NULL)
                {
                        printf("[TOOL_FAILURE] strdup %s\n", strerror(errno));
                        exit(-1);
                }

	real_do_debug_vaddr = get_symbol("do_debug", file_type, sympath);
	if (real_do_debug_vaddr == 0)
	{
		printf("[+] Unable to locate symbol: do_debug\n");
		exit(0);
	}

	printf("[+] Located symbol: do_debug @ 0x%x\n", real_do_debug_vaddr);

	dsc_vaddr = get_symbol("debug_stack_correct", file_type, sympath);
	if (dsc_vaddr == 0)
	{
		printf("[+] Unable to locate symbol: debug_stack_correct\n");
		exit(0);
	}

	printf("[+] Located symbol: debug_stack_correct @ 0x%x\n", dsc_vaddr);

	printf("[+] Opening path: %s\n", kpath);

	if ((kfd = open(kpath, O_RDONLY)) == -1)
	{
		printf("[TOOL_FAILURE] open %s\n", strerror(errno));
		exit(-1);
	}
	
	if (readkmem(code, dsc_vaddr, sizeof(code)) < 0)
	{
		printf("[TOOL_FAILURE] readkmem failed %s\n", strerror(errno));
		exit(-1);
	}
	
	/* initialize */
	do_debug_off = 0;

	/* Locate call do_debug. We must get It's offset so we can figure out */
	/* If it correlates to the memory address in /proc/kallsyms */ 
	for (p = code, i = 0; i < 64; i++)
	{
		if (p[i] == 0xe8)
		{	
			do_debug_off = *(unsigned long *)&p[i + 1];
			do_debug_vaddr = (dsc_vaddr + i) + do_debug_off + 5;
			printf("[+] do_debug offset found: %d\n", do_debug_off);
			printf("[+] do_debug vaddr resolved: 0x%x\n", do_debug_vaddr);
			break;
		
		}
	}

	if (!do_debug_off)
	{
		printf("[+] Unable to locate call instruction within INT1 Handler\n");
		exit(0);
	}

	printf("[+] Checking to see if INT1 handler has been patched to call an evil do_debug()\n");

	if (do_debug_vaddr != real_do_debug_vaddr)
		printf("[+] [ALERT_TO_DAEMON] INT 1 handler appears to be illegally patched with a call to an evil debug handler\n"
		       "The correct do_debug should be at 0x%x, but instead exists at 0x%x\n"
		       "this means that a Debug Register rootkit may be installed!\n", real_do_debug_vaddr, do_debug_vaddr);
	else
		printf("[+] System seems to have a healthy INT 1 handler, this means that a Debug Register rootkit is probably not installed\n");
	
	fflush(stdout);
	exit(0);
}
	
	
