/*
 * KMP - Kernel Memory Patcher (C) Innosecc.com
 * Simple runtime kernel patcher, unlikely that write access to /dev/kmem is available  
 * but if it is, this tool can be used in conjunction with a tool like KmemStat very nicely
 * 
 * NOTE: /dev/kmem is usually writeable in FreeBSD 
 *
 * Author: Ryan O'Neill
 * <ryan@innosecc.com>
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/types.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <getopt.h>
#include <string.h>

#include <elf.h>
#include <sys/mman.h>

#define PAGE_SIZE 4096

int main(int argc, char **argv)
{
	int c = 0, i = 0, fd;
	long *code;
	unsigned char *code_str, *mem;
	unsigned long vaddr;
	int code_len;

	usage:
	if (argc < 3)
	{
		printf("Usage: %s <virtual address> <code string\n", argv[0]);
		printf("i.e - %s 0xc030112f b8 ff de 70 80 ff e0\n", argv[0]);
		exit(0);
	}
	
	if (strlen(argv[1]) != 10)
		goto usage;

	if (argv[1][0] != '0' && tolower(argv[1][1]) != 'x')
		goto usage;
	
	vaddr = strtoul(argv[1], NULL, 16);
	code = (long *)malloc(sizeof(long) * (argc - 1));
	code_str = (unsigned char *)malloc(argc-1); 

	argc -= 2;
	argv += 2;
	
	while (argc--)
	       code[c++] = strtol(*argv++, NULL, 16);
	code_len = c - 1;

	printf("Patching the kernel ->\n"
	       "Offset Vaddr: %x\nCode: ", vaddr);

	while (i < c)
		printf("%.2x ", code[i++]);
	puts("\n");
	
	i = 0;
	while (i < c)
		code_str[i] = (unsigned char)code[i++];
	
	
	if ((fd = open("/dev/kmem", O_WRONLY)) == -1)
	{
		perror("open kmem");
		exit(-1);
	}
	mem = mmap(0 , code_len + (PAGE_SIZE - (vaddr & (PAGE_SIZE - 1))), PROT_WRITE, MAP_SHARED, fd, vaddr & (~(PAGE_SIZE-1)));
	if (mem == MAP_FAILED)
	{
		perror("mmap");
		goto end;
	}

	memcpy(mem, code_str, code_len+1);
	msync(mem, code_len, MS_SYNC);

	munmap(mem, vaddr+code_len);
	
	end:
	close(fd);
	free(code);
	free(code_str);
	       
	printf("Kernel patching complete!\n");
	exit(0);
}
