/*
 * KMTD (C) Innosecc.com 2009. 
 * The Kernel malware tracker Daemon. This software abides by the KMTD protocol
 * as specified in KMTD-PROTOCOL file.
 * Author of Daemon: Ryan@Innosecc.com 
 * Author of Client: Sean@Innosecc.com
 */


#define SSL_ENABLED 1	

#include "kmtd.h"

/* prototype here for now,later in common.h */
int pidfile_make(const char *progname);

struct opts opts;
struct tool_list tool_list;

char global_alert_buffer[8192 * 8]; /* Processed data output when kernel malware is detected */
char global_stats_buffer[8192 * 8]; /* statistics about latest scan, and results */
char hostname[255];
uint32_t host_ip;
char host_OS[255];

char sysmap_path[255], kernimg_path[255];
char *email[255];
int email_count = 0;

/* digest for System.map */
uint8_t sysmap_digest[16];
/* digest for uncompressed kernel image */
uint8_t kernelf_digest[16];

/* These lengths correspond to bufs above */
unsigned int alert_len = 0;
unsigned int stats_len = 0;

struct config_info config;
int interactive = 0;
int t_flag = 0;

int SendAlerts = 0;
char global_zip_passwd[32];
char mailutil[255];

/* The number of different tools that the daemon calls */
int ACTIVE_TOOL_COUNT = 0;
int ALREADY_BOUND = 0;

int port = KMTD_PORT;

int global_sig_ret = 0;

int tool_status[5];

int ActiveSession = 0;

/* A couple of global threads */
pthread_t tail_thread;
pthread_attr_t detached_attr;

static int process_tool(rk_tool_opts *tool, int mode)
{
	static int ToolExecCount = 0;
	int pid, status, ret = 0;
	int tokens, i;
	int pd[2];
	char buf[512], syslog_msg[4096] = {0};
	char *sb; /* send buffer */
	char *ptr;
	char **xargs; /* execv arg vector */
	char tmp[512];
	int tool_failure = NO;

	memset(global_alert_buffer, 0, sizeof(global_alert_buffer));

	if (opts.debug && interactive)
	{
		printf("[+] process_tool()\n"
		       "Tool name: %s\n"
		       "Exec path: %s\n"
		       "Log File : %s\n"
		       "OptionStr: %s\n"
		       "Linux OS : %c\n"
		       "FBSD OS  : %c\n",
		tool->tool_name, tool->exec_path, tool->logfile_name,
	        tool->option_str, tool->supported_by_linux ? 'Y' : 'N',
		tool->supported_by_freebsd ? 'Y' : 'N');
	}
	if (pipe(pd) < 0)
	{
		KMTLog_err("pipe() error: %s", strerror(errno));
		kmtd_panic(MISC_PANIC);
	}
	
	/* Increment Tool execution count by 1 */
	ToolExecCount++;
	/* Once we've executed all of the tools clear out */
	/* the global stats buffer so that it can get fresh data */
	if (ToolExecCount > ACTIVE_TOOL_COUNT) 
	{
		ToolExecCount = 0;
		memset(global_stats_buffer, 0, stats_len);
		stats_len = 0;
	}
	
	/* Create argument vector for execv */	
	sprintf(tmp, "%s %s", tool->tool_name, tool->option_str);
        tokens = exargs(&xargs, " ", tmp);
	
	if ((pid = fork()) < 0)
	{
		KMTLog_err("fork() error: %s\n", strerror(errno));
		kmtd_panic(MISC_PANIC);
	}
	
	/* Execute tool and handle pipes */
	if (!pid)
	{
		close(pd[0]);
		dup2(pd[1], STDOUT_FILENO);
		close(pd[1]);
		
		ret = execv(tool->exec_path, xargs); 
		if (ret == -1)
		{
			KMTLog_err("exeve() error: %s\n", strerror(errno));
			NotifyUsers("Error executing tool [execve() %s]\n", strerror(errno));
		}
	}
	/* Get execution status of child */
	wait(&status);
	if (WIFEXITED(status))
		tool_status[tool->toolID] |= CLEAN_EXIT;
	else
	{
		tool_status[tool->toolID] |= CRASH_EXIT;
		if (WIFSIGNALED(status))
			tool_status[tool->toolID] |= SIGNL_EXIT;
		if (WCOREDUMP(status))
			tool_status[tool->toolID] |= CORED_EXIT;
	}
		
			
	close(pd[1]);
	alert_len = read(pd[0], global_alert_buffer, sizeof(global_alert_buffer));
	
	if (opts.debug)
		printf("[+] alert_len: %d\n", alert_len);
	
	/* ------------------- IMPORTANT ------------------*/
	/* Does the tool fail? i.e malloc() failure */
	if (memmem(global_alert_buffer, sizeof(global_alert_buffer), "[TOOL_FAILURE]", 14))
	{
		close(pd[0]);
		snprintf(buf, sizeof(buf), "[%s] may be having functionality issues: %s\n", tool->tool_name, global_alert_buffer);
		KMTLog_err("%s", buf);
		/* Not sure why I had this strncmp here */
		//if (strncmp(global_alert_buffer, "[TOOL_FAILURE]", 14) == 0)
		//{
			tool_failure = YES;
			if (ActiveSession)
			{
				if (mode == SINGLE_MODE)
					SendMsg("-TOOL_FAILURE %s SINGLE %d %s\n",tool->tool_name, alert_len - 14, (global_alert_buffer + 14));
				else
					SendMsg("-TOOL_FAILURE %s LOOP %d %s\n", tool->tool_name, alert_len - 14, (global_alert_buffer + 14));
			}
		//}
		// don't see why I put this memset here  
		//memset(global_alert_buffer, 0, sizeof(global_alert_buffer));

	}

	memcpy(&global_stats_buffer[stats_len], global_alert_buffer, alert_len);
       	stats_len += alert_len;
	
	close(pd[0]);
	free(xargs);
	/* ---------------------- IMPORTANT -----------------------*/
        /* The alerts len will hold only the output from a single tool at a time */
	if (memmem(global_alert_buffer, sizeof(global_alert_buffer), ALERT_DAEMON, strlen(ALERT_DAEMON)))
        {
		if (opts.debug)
			printf("[+] SendAlerts: %d\n", SendAlerts);
		if (SendAlerts)
			ReportIntrusion(tool->toolID); 	
		snprintf(syslog_msg, sizeof(syslog_msg),  "Discovered malware; With module [%s]. View %s for more information\n", tool->tool_name, tool->logfile_name);
		syslog(LOG_DAEMON, "%s", syslog_msg);
        	KMTLog(tool->tool_name, opts.logpath);
	 	Write2Log(tool->logfile_name);
		/* Alert GUI unless we're in SINGLE_MODE */
		if (mode == MULTI_MODE)
		{
			if (opts.debug)
				printf("[+] process_tool() is returning ALERT_GUI\n");
			return ALERT_GUI;
		}
	}
	
	/* If GUI is connected and in single mode */
	if (mode == SINGLE_MODE)
	{
		if (tool_status[tool->toolID] == CRASH_EXIT)
			SendMsg("-ERR TOOL_CRASHED (UNCLEAN EXIT)\n");
		else
		if (tool_status[tool->toolID] == (CRASH_EXIT | SIGNL_EXIT))
			SendMsg("-ERR TOOL_CRASHED (EXIT ON SIGNAL)\n");
		else
		if (tool_status[tool->toolID] == (CRASH_EXIT | SIGNL_EXIT | CORED_EXIT))
			SendMsg("-ERR TOOL_CRASHED (EXIT ON COREDUMP)\n");
		else
		{
			if (tool_failure == NO)
				SendMsg("+OK %d %s\n", alert_len, global_alert_buffer);
		}	
	}

	/* Write to tool specific log file */
	Write2Log(tool->logfile_name);
	return 0;

}

/* This is only for sending alerts to the GUI (Not emails) */
void SendAlert(char *tool_name)
{
	char *buf, *p;
	char tmp[16], msg_alert[strlen(MSG_ALERT) + 1];

	if (ActiveSession == 0)
		return;

	if ((buf = (char *)malloc(alert_len + strlen(tool_name) + strlen(MSG_ALERT) + 20)) == NULL)
	{
		syslog(LOG_ERR, "Exiting on failure in malloc: %s\n", strerror(errno));
		KMTLog_err("kmtd failed with malloc() in SendAlert()");
	        kmtd_panic(MISC_PANIC);
	}
	
	sprintf(buf, "%s %s %d %s\n", MSG_ALERT, tool_name, alert_len, global_alert_buffer);
	if (send_wrapper(buf, strlen(buf)) == -1)
	{
	    //TODO: do what?
	 }

}
	
static int process_tools()
{
	int ret;
	if (opts.debug)
		printf("[+] Process_tools()\n");
	
	memset((int *)tool_status, 0, sizeof(tool_status));
	
	if (tool_list.kmemstat && config.kmemstat.enabled)
	{
		ret = process_tool(&config.kmemstat, 0);
       		if (ret == ALERT_GUI)
        		SendAlert("kmemstat");
	}

	if (tool_list.ksymstat && config.ksymstat.enabled)
	{
        	ret = process_tool(&config.ksymstat, 0);
        	if (ret == ALERT_GUI)
                	SendAlert("ksymstat");
	}
	
	if (tool_list.ktraq && config.ktraq.enabled)
	{
        	ret = process_tool(&config.ktraq, 0);
        	if (ret == ALERT_GUI)
                	SendAlert("ktraq");
	}

	if (tool_list.dps && config.dps.enabled)
	{
        	ret = process_tool(&config.dps, 0);
        	if (ret == ALERT_GUI)
                	SendAlert("dps");
	}

	if (tool_list.dr_chk && config.dr_chk.enabled)
	{
		ret = process_tool(&config.dr_chk, 0);
                if (ret == ALERT_GUI)
                        SendAlert("dr_chk");
	}
}

static int SendStats()
{
	char *buf, *p;
        char tmp[16], msg_ok[strlen(MSG_OK) + 1];

        if ((buf = (char *)malloc(stats_len + strlen(MSG_OK) + 20)) == NULL)
        {
		KMTLog_err("malloc failure: %s\n", strerror(errno));
        	kmtd_panic(MISC_PANIC);
	}

	sprintf(buf, "%s %d %s", MSG_OK, stats_len, global_stats_buffer);

        if (send_wrapper(buf, strlen(buf)) == -1)
	{
		/* EPIPE means peer disconnected */
		 if (errno == EPIPE)
			return PEER_DISCONNECT;
                 else
			return SOCKET_ERR;
	}
	
	return 0;
}
/* returns -1 on failure */
// OR PEER_DISCONNECT (-2)
// OR SOCKET_ERR (-3)
// OR 0 for success
int SendKimages()
{
	char **kimage_list;
	char *kbuf;
        char tmp[4096] = {0};

        int len = 0, count = 0;
        int i;

	kimage_list = (char **)get_kimage_paths(&count, &len);
	
	kbuf = alloca(len + strlen(MSG_OK) + count + 1);
	/*
	if ((kbuf = (char *)malloc(len + strlen(MSG_OK) + count + 1)) == NULL)
        {
                KMTLog_err("fatal error in SendAlert(): malloc");
	        exit(-1);
        }
	*/
	if (opts.debug)
		printf("SendKimages() count: %d len: %d\n", count, len);

	/* Didn't find any kernel images */
	if (count == 0 || kimage_list == NULL) 
        {
                if (send_wrapper("+OK\n", 4) == -1)
                {
                        if (errno == EPIPE)
                                return PEER_DISCONNECT;
                        else
                                return SOCKET_ERR;
                }
                goto done;
        }

	/* We found 1 kernel image */
	if (count == 1)
        {
                sprintf(kbuf, "+OK %s\n", kimage_list[0]);
		if (opts.debug)
			printf("SendKimages sending string: %s\n", kbuf);

		if (send_wrapper(kbuf, strlen(kbuf)) == -1)
                {
                        if (errno == EPIPE)
                                return PEER_DISCONNECT;
                        else
                                return SOCKET_ERR;
                }
                goto done;
        }

	
	if (opts.debug)
                printf("[+] Found %d uncompressed kernel images\n", count);

        /* We found multiple kernel images */
        sprintf(kbuf, "+OK ");
        for (i = 0; i < count; i++)
        {
                if (i == count - 1)
                        strncat(tmp, kimage_list[i], 255);
                else
                {
                        strncat(tmp, kimage_list[i], 255);
                        strcat(tmp, ",");
                }
        }

        strncat(kbuf, tmp, len + count - 1);
        strcat(kbuf, "\n");
	
	if (opts.debug)
		printf("[+] SendKimage(): Sending %s\n", kbuf);

        if (send_wrapper(kbuf, strlen(kbuf)) == -1)
        {
                /* EPIPE means peer disconnected */
                 if (errno == EPIPE)
                        return PEER_DISCONNECT;
                 else
                        return SOCKET_ERR;
        }

        done:
	//free(kbuf);
	if (count > 0)
	{
		if (opts.debug)
			printf("Kimage count %d len %d, freeing kimage_list\n", count, len);
		free(kimage_list);
	}
        return 0;
}

/* Get list of System.map locations */
int SendSYSMAPS()
{
	char **smap_list;
	
	char *buf;
	char tmp[4096] = {0};

	int len = 0, count = 0;
	int i;

	/* Get System.map paths */
	smap_list = get_sysmap_paths(&count, &len);

	/*
        if ((buf = (char *)malloc(len + strlen(MSG_OK) + count + 1)) == NULL)
        {
                perror("fatal error in SendAlert(): malloc");
                exit(-1);
        }
	*/
	buf = alloca(len + strlen(MSG_OK) + count + 1);
	printf("buf: %s\n", buf);
	/* We didn't find any System.map files */
	if (count == 0)
	{
		if (send_wrapper("+OK\n", 4) == -1)
		{
			if (errno == EPIPE)
                        	return PEER_DISCONNECT;
                 	else
                        	return SOCKET_ERR;
		}
		goto done;
	}
	
	/* We found only 1 file */
	if (count == 1)
	{
		sprintf(buf, "+OK %s\n", smap_list[0]);
		if (send_wrapper(buf, strlen(buf)) == -1)
		{
                        if (errno == EPIPE)
                                return PEER_DISCONNECT;
                        else
                                return SOCKET_ERR;
                }

		goto done;
	}

	if (opts.debug)
		printf("[+] Found %d System.map locations\n", count);

	/* We found multiple System.map files */
	sprintf(buf, "+OK ");
	for (i = 0; i < count; i++)
	{
		if (i == count - 1)
			strncat(tmp, smap_list[i], 255);
		else
		{
			strncat(tmp, smap_list[i], 255);
			strcat(tmp, ",");
		}
	}
	
	strncat(buf, tmp, len + count - 1);
	strcat(buf, "\n");

	if (opts.debug)
		printf("[+] SendSYSMAPS(): Sending %s\n", buf);

	if (send_wrapper(buf, strlen(buf)) == -1)
        {
                /* EPIPE means peer disconnected */
                 if (errno == EPIPE)
                        return PEER_DISCONNECT;
                 else
                        return SOCKET_ERR;
        }

	done:
	if (count > 0)
		free(smap_list);
        return 0;
	
}
/* This is for when the daemon sends the initial */
/* UPDATE_CONF to the client so it has the current */
/* configuration settings */
/* return 0 on success, -1 or PEER_DISCONNECT, or SOCKET_ERR on fail */
int SendUpdateConf()
{
	struct stat st;
	int fd;
	char *buf, *p, *mem;
	char tmp[16];
	int c;

	if ((fd = open(opts.config, O_RDONLY)) < 0)
	{
		perror("open");
		return -1;
	}
	
	if (fstat(fd, &st) < 0)
	{
		perror("fstat");
		return -1;
	}
	
	if ((buf = (char *)malloc(st.st_size + strlen(MSG_UPDATE_CONF) + 26)) == NULL)
	{
		perror("malloc");
		return -1;
	}

	mem = mmap(NULL, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
	if (mem == MAP_FAILED)
	{
		perror("mmap");
		return -1;
	}

	int len;
	sprintf(buf, "%s %d ", MSG_UPDATE_CONF, st.st_size);
	len = strlen(buf);

	memcpy(&buf[len], mem, st.st_size);
	len += st.st_size;
	buf[len++] = '\n';//NOTE gui is expecting this finishing newline as per protocols

	if (send_wrapper(buf, len) == -1)
	{
		if (errno == EPIPE)
		{
			close(fd);
			munmap(mem, st.st_size);
			return PEER_DISCONNECT;
		}
		else
		{
			KMTLog_err("kmtd failed on send_wrapper()");
		    	return -1;
		}
	}

	close(fd);
	munmap(mem, st.st_size);
	
    return 0;
}
	
	
/* We received UPDATE_CONF and proper config data */
/* Lets write the config, then possibly execute tools */
/* after reloading */
// data is null terminated C string of xml  - Sean
int ConfirmConfig(char *data, int do_execution)
{
	/* TODO: we should ensure it actually succeeded in writing the conf file before sending the OK - Sean */
	char resp[20];
	char *p, *datap;

	snprintf(resp, sizeof(resp), "%s\n", MSG_OK); /* was missing the \n */
        if (send_wrapper(resp, strlen(resp)) == -1)
        {
	   	KMTLog_err("send_wrapper() failure: %s\n", strerror(errno));
        	/* if EPIPE, then peer disconnected */
                if (errno == EPIPE)
			return PEER_DISCONNECT;
                else
			return SOCKET_ERR;
        }

	if (opts.debug && interactive)
		printf("[+] Writing config.\n");

	/* if client sent the <xml> element and neglected a '\n', add it.
	 * Java may or may not send it. This is for asthetic purposes */
	if((p = strstr(data, "><root_element>")) != NULL)
	{
	 	size_t len;
		char *databuf;
	    	char *q;

		if (opts.debug && interactive)
	    		printf("[!] Correcting indentation on received conf file\n");

		len = strlen(data);
		/* +2, one for the \0, one for the new newline character */
		databuf = alloca(len + 2);
		*databuf = '\0';
		/* (p-data+1) is len of the <?xml tag> */
		strncat(databuf, data, p - data + 1);
		strcat(databuf, "\n");
                /* (p + 1) is the opening '<' of '<root_element>' */
                strcat(databuf, p + 1);
		datap = databuf;	
	} 
	else 
		datap = data;

        WriteConfig(opts.config, datap);
        Read_KMD_Config(opts.config, &config);
 	
	memset(sysmap_path, 0, sizeof(sysmap_path));
	memset(kernimg_path, 0, sizeof(kernimg_path));

	strncpy(sysmap_path, config.system_map, sizeof(sysmap_path));
        sysmap_path[sizeof(sysmap_path) - 1] = '\0';

        strncpy(kernimg_path, config.kernel_image, sizeof(kernimg_path));
        kernimg_path[sizeof(kernimg_path) - 1] = '\0';

        /* Get initial fingerprints on System.map and vmlinux/kernel signatures */
        global_sig_ret = FPSigs(config.system_map, config.kernel_image);
	
	if (do_execution == EXECUTE_TOOLS)
	/* Execute tools after new config has been loaded */
		process_tools();	
	else
		return 0;		

}

jmp_buf env;
int AlreadyBound = 0;

void sighandle(int sig)
{
        FILE *fd;

        char time_buf[255];
        time_t clock;
        struct tm *tm;

        /* No real need for error checking if we caught SIGKILL lol */
       
	switch(sig)
	{
		case SIGINT:
			if ((fd = fopen("/var/log/kmt/kmtd.log", "a+")) == NULL)
				exit(-1);
			
 		        time (&clock);
        		tm = gmtime (&clock);
        		setlocale (LC_TIME, "C");
        		strftime (time_buf, sizeof (time_buf), "%A, %B %e, %Y, %H:%M:%S", tm);
        		fprintf(fd, "%s Caught SIGINT\n", time_buf);
        		fflush(fd);
        		fclose(fd);
			exit(0);
			break;
		case SIGHUP:
		        longjmp(env, sig);	
			break;
	}
}

int main(int argc, char **argv)
{
	int c, ret;
	int bindfd, sin_size, tail_in_progress = 0;
	extern char *optarg;
	struct timeval tv, tv_timeout;
	unsigned int saved_sec;
	uint8_t InitTime = 0;
	uint8_t WaitingOnTimeOut = 0, PrevSessEnded = 0;
	unsigned int elapsed_time;
	unsigned int timeout_start_time;
	unsigned int bytes, next_bytes;
	int status, sm;
	FILE *logfd;
	int N, N_len, i, j, fpret;
        int kmtd_pid, kmtd_pgid;
	struct sigaction act;
        sigset_t set;
	
	char *data, *cli_ip;
	uint8_t *p;

	struct tail_args tail_args_data;

	opts.debug = 0;
	opts.verbose = 0;
	opts.custom_log_path = 0;
	opts.custom_conf_path = 0;
 	
	memset(&tool_list, 0, sizeof(struct tool_list));

	goto GetOpts;
       
        Usage:
        {
		printf("\nKMT Daemon (C)\tInnovative Security Concepts, Inc. 2009\n"
		"\t\thttp://innosecc.com\n\n"
	       "Usage: %s [-c <config_path>] [-l <logfile_path>] [-vdiht]\n"
	       "[-l] <logfile_path>\n"
	       "[-c] <config_path>\n"
	       "[-p] <port>\n"
	       "[-v] Verbose output\n"
	       "[-d] Debug output\n"
	       "[-t] Test alerts\n"
	       "[-i] Interactive mode (For debug)\n"
	       "[-h] Help\n\n", argv[0]);
		exit(0);
	}
	
	GetOpts:
	while ((c = getopt(argc, argv, "ihdvl:c:tp:")) != -1)
        {
                switch(c)
		{
			case 'd':
				opts.debug++;
				printf("[!] Debug mode enabled\n");
				break;	
			case 'v':
				opts.verbose++;
				break;
			case 'l':
				opts.custom_log_path++;
				if ((opts.logpath = strdup(optarg)) == NULL)
				{
					perror("Fatal: strdup");
					exit(-1);
				}
				break;
			case 'c':
				opts.custom_conf_path++;
				if ((opts.config = strdup(optarg)) == NULL)
                                {
                                        perror("Fatal: strdup");
                                        exit(-1);
                                }
				break;
			case 'h':
				goto Usage;
				break;
			case 'i':
				interactive++;
				break;
			case 't':
				t_flag++;
				break;	
			case 'p':
				port = atoi(optarg);
				break;
		 }
	}
	
	/* Use default log and config paths if not set */  
	if (!opts.custom_log_path)	
		if ((opts.logpath = strdup(DEFAULT_LOG_PATH)) == NULL)
                {
                	perror("Exiting on failure, strdup");
                        exit(-1);
                }

	if (!opts.custom_conf_path)
                if ((opts.config = strdup(DEFAULT_CONF_PATH)) == NULL)
                {
                        perror("Exiting on failure, strdup");
                        exit(-1);
                }
	
	if ((logfd = fopen(opts.logpath, "w+")) == NULL)
	{
		perror("Unable to open logfile: fopen");
		exit(-1);
	}

	if (interactive)
		logfd = stdout;

	if (opts.debug)
		fprintf(logfd, "[!] Reading in config file\n");


	/* If we get a SIGHUP, we start back at this point */
	/* hence the setjmp(env) */
	setjmp(env);

	set_host_values();

	if (Read_KMD_Config(opts.config, &config) == -1)
	{
		fprintf(stdout, "Unable to start daemon, config cannot be read\n");
		exit(-1);
	}
	
	if(t_flag > 0)
	{
	    //NOTE: t flag doesnt care about send_alerts in the config
		SendAlerts = 1;
		ReportIntrusion(TOOL_T_FLAG);	
		exit(EXIT_SUCCESS);
	}

	/* Temporarily made these paths global for the sake of */
	/* kmtd_panic() */
	strncpy(sysmap_path, config.system_map, sizeof(sysmap_path));
	sysmap_path[sizeof(sysmap_path) - 1] = '\0';
	
	strncpy(kernimg_path, config.kernel_image, sizeof(kernimg_path));
        kernimg_path[sizeof(kernimg_path) - 1] = '\0';

	/* Get initial fingerprints on System.map and vmlinux/kernel signatures */
	global_sig_ret = ret = FPSigs(config.system_map, config.kernel_image);

	if (ret == ERR_FP_SYSMAP)
		fprintf(logfd, "[!] Unable to get MD5 digest for %s (Tools that use this may fail)\n", config.system_map);
	else
	if (ret == ERR_FP_KERNIMG)
		fprintf(logfd, "[!] Unable to get MD5 digest for %s (Tools that use this may fail)\n", config.kernel_image);
	

	if (opts.debug && !interactive)
		fprintf(logfd, "[!] Daemonizing\n");

	int noclose;

	if (opts.debug)
		noclose = 1;
	else
		noclose = 0;

	/* If we are in interactive mode we don't even daemonize */
	if (!interactive)
		if (daemon(1, noclose) == -1) 
       		{
               		perror("Fatal error in daemonizing: daemon()");
               		exit (-1);
       		}
	
	/* now that went daemon and have new pid, create the pid file - sean */
	if(pidfile_make("kmtd") == -1)
	{
		fprintf(logfd, "Could not make pidfile, exiting\n");
		exit(-1);
	}

	//NOTE: best called after daemon()
	//proccess id of calling process used
	// kmtd pid becomes group id for kmtd daemon and all descendants. 
	setpgid(0, 0);
	kmtd_pgid = kmtd_pid = getpid();

	signal(SIGINT, SIG_IGN);
	signal(SIGTERM, SIG_IGN);
       
	act.sa_handler = (void (*)())sighandle;
        sigemptyset (&act.sa_mask);
        act.sa_flags = 0;
        sigaction (SIGINT, &act, NULL);
  	sigaction (SIGHUP, &act, NULL);
	sigemptyset (&set);
        sigaddset (&set, SIGINT);
	sigaddset (&set, SIGHUP);

	if ((data = (char *)malloc(4096)) == NULL)
	{
		perror("Existing on failure, malloc");
		exit(-1);
	}

	if (opts.debug)
		fprintf(logfd, "[!] Listening on port %d\n", port);

	
	if (AlreadyBound == 0)
	{
		/* Bind addr to socket */
		bindfd = bind_connection();
		AlreadyBound = 1;
	}

 	/* listen backlog of 1 since there is only one possible connection at a time */
	if (listen(bindfd, 1) < 0)
	{
		fprintf(logfd, "Fatal err listen(): %s ", strerror(errno));
		exit(-1);
	}


	fflush(logfd);
	InitTime = 1;

	/* THE FOLLOWING LOOP IS THE CONNECTION HANDLING CODE */
	/* This deals with the entire KMT protocol */ 
	PrevSessEnded = 1;
	for (;;)
	{
		/* label 1 for outter loop */
		WAIT_ACCEPT:
		if (PrevSessEnded)
		{
			if ((cli_ip = accept_wrapper(bindfd, SSL_ENABLED)) == NULL)
        		{
				if (errno == EAGAIN || errno == EWOULDBLOCK)
					goto BEGIN_LOOP;

			    	goto WAIT_ACCEPT;
        		}
			else
				ActiveSession = 1;
			/* do authentication */
			
			bytes = recv_wrapper(data, 4096);
			if (bytes == -1)
				goto END_SESSION;
			if(strncasecmp(data, "AUTH ", strlen("AUTH ")))
			{	
				fprintf(logfd, "[!] Failed authentication from %s\n", cli_ip);
				SendMsg("-ERR Invalid Command\n");
				goto END_SESSION;
			}
			else 
			{
				p = strchr(&data[5], '\n');
				if (p)
					*p = '\0';
				p = strchr(&data[5], '\r');
				if (p)
					*p = '\0';
				
			  	if(strcmp(&data[5], config.jkmt_passwd))
			      	{
					SendMsg("-ERR Invalid Password\n");

 					fprintf(logfd, "[!] Failed authentication from %s\n", cli_ip);
					goto END_SESSION;
				}
			}


 			fprintf(logfd, "[!] Successful authentication from %s\n", cli_ip);

			/* Send initial UPDATE_CONF to client */
			SendUpdateConf();
			PrevSessEnded = 0;
		}
		/* label 2 for outter loop */
		BEGIN_LOOP:
		if (opts.debug)
			fflush(logfd);
		
		/* sleep(N) This takes 95% CPU utilization down to about 0.5% */
		PAUSE_A_MOMENT;

		/* Start Counting */
		if (InitTime)
		{
			gettimeofday(&tv, NULL);
			saved_sec = tv.tv_sec;
			InitTime = 0;
		}
		else
		{
			/* Have we reached the interval time? */
			/* If so, then execute tool modules to */
			/* scan for kernel malware */
			gettimeofday(&tv, NULL);
			elapsed_time = tv.tv_sec - saved_sec;
			if (elapsed_time == config.time_interval)
			{	
				/* execute tools and analyze */
				
				fpret = VerifySigFP(config.system_map, config.kernel_image);
				//fpret = 0; /* temporary workaround since VerifySIG is broken */
				switch(fpret)
				{
				case ERR_FP_SYSMAP:
					/* If we got this then perhaps the config */
					/* was corrupted and the system.map path is */
					/* no longer valid. */
					NotifyUsers(
					"Unable to MD5 fingerprint System.map file. Please make sure the kmt.conf file\n"
					"has not been corrupted, and confirm that its <system_map> entry matches the\n"
					"location of System.map. We suggest using the GUI to reconfigure this\n");
					break;
					// (LETS NOT PANIC HERE ANYMORE) kmtd_panic(MISC_PANIC);
				case ERR_FP_KERNIMG:
					/* If we got this then perhaps the config */
                                        /* was corrupted and the vmlinux/kernel path is */
                                        /* no longer valid. */
                                        NotifyUsers(
                                        "Unable to MD5 fingerprint kernel image. Please make sure the kmt.conf file\n"
                                        "has not been corrupted, and confirm that its <kernel_image> entry matches the\n"
                                        "location of the kernel image (i.e vmlinux). We suggest using the GUI to reconfigure this\n");
					break;
                                       // (LETS NOT PANIC HERE ANYMORE) kmtd_panic(MISC_PANIC);
				case ERR_SYSMAP_MISMATCH:
					/* The System.map file has been modified, or the wrong file is in place */
					NotifyUsers(
					"The System.map file has been modified since KMT's last malware scan (%d seconds ago)\n \
					As a security measure KMT is shutting down until the reason for this change has been\n  \
					determined. Keep in mind that an intruder may modify System.map to reflect the memory\n  \
					changes that transpire as a result of the kernel rootkit.\n", config.time_interval);
					kmtd_panic(SYSMAP_PANIC);
				case ERR_KERNIMG_MISMATCH:
					/* The System.map file has been modified, or the wrong file is in place */
                                        NotifyUsers(
                                        "The uncompressed kernel image has been modified since KMT's last malware scan (%d seconds ago)\n"
                                        "As a security measure KMT is shutting down until the reason for this change has been\n"
                                        "determined. Keep in mind that an intruder may modify the kernel image to reflect the memory\n"
                                        "changes that transpire as a result of the kernel rootkit.\n", config.time_interval);
                                        kmtd_panic(KERNIMG_PANIC);

					
				}	
				
					
				printf("Calling process_tools\n");	
				process_tools();
				InitTime = 1;
			}
		}
		if (PrevSessEnded)
			goto WAIT_ACCEPT;	


			bytes = recv_wrapper(data, 4096);
			if (opts.debug && interactive)
			if (bytes == -1)
				printf("bytes: %d, errno: %s\n", bytes, strerror(errno));

			/* 0 means peer has performed an orderly shutdown */
			if (bytes == 0 || bytes == -1)
				goto END_SESSION;

			if (opts.debug)
				fprintf(logfd, "[!] Bytes read: %d ->\n", bytes);
			
			if (opts.debug && bytes > 0)
			{
				for (j = 0; j < bytes; j++)
					printf("%c", data[j]);
				printf("\n");
			}
			/* HANDLE OUR PROTOCOL INTERPETATION */
			if (strncasecmp(data, MSG_UPDATE_CONF, strlen(MSG_UPDATE_CONF)) == 0)
			{
				p = &data[strlen(MSG_UPDATE_CONF) + 1 /*one space*/]; /* Should be N (to describe config byte len) */
				/* Calculate the value of N, as well as how many chars it is (N_len) */
				for (N = 0, N_len = 0; *p != 0x20; p++, N_len++)
					N = 10 * N + (*p - '0');
				if (opts.debug)
					fprintf(logfd, "[!] Received UPDATE_CONF, Incoming config is %d bytes (N_len is: %d)\n", N, N_len);
				/* Check to make sure our config data */
				/* is equal to config_len */
				//TODO: check spaces and newline are where they should be and number is all digits.
				bytes = bytes - strlen(MSG_UPDATE_CONF) - N_len - 2 /*two spaces */ - 1 /*newline termining cmd */;
				if (bytes != N)
				{
					/* This means the config text */	
					/* is not equal the size that N said */
					/* it should be, according to our protocol */
					/* Unless the data hasn't fully arrived */
					for (;;)
					{
							next_bytes = recv_wrapper(data, 4096);
							if (next_bytes == 0)
								goto END_SESSION;	
							if ((next_bytes + bytes) != N) 
							{
								/* If the number of bytes still don't match */
								/* Send an error */
					 			if (send_wrapper(MSG_ERROR_LEN_MISMATCH, strlen(MSG_ERROR_LEN_MISMATCH)) == -1)
								{
									 /* if EPIPE, then peer disconnected */
									 if (errno == EPIPE)
                                                                                goto END_SESSION;
                                                                         else
									 {
									 	syslog(LOG_ERR, "Existing on failure in send: %s\n", strerror(errno));
                                                                                exit(-1); 
									 }
								}
								/* Goto outter loop (continue no work) */
								goto BEGIN_LOOP;
							}
							else
							{
								/* We've succeeded in getting the full config */
								/* But we had to read the socket twice to get */
								/* everything because all data wasn't in the */
								/* tcp buffer the first time around */

								char *datap = &data[strlen(MSG_UPDATE_CONF) + 2 /* spaces */ + N_len];
								datap[N] = '\0'; //cut off the \n
								ConfirmConfig(datap, EXECUTE_TOOLS);
									
								 /* Get initial fingerprints on System.map and vmlinux/kernel signatures */
								/*
								global_sig_ret = ret = FPSigs(config.system_map, config.kernel_image);

								if (ret == ERR_FP_SYSMAP)
                							fprintf(logfd, "[!] Unable to get MD5 digest for %s (Tools that use this may fail)\n", config.system_map);
        							else
        							if (ret == ERR_FP_KERNIMG)
                							fprintf(logfd, "[!] Unable to get MD5 digest for %s (Tools that use this may fail)\n", config.kernel_image);
								*/
								
							}
						if (!WaitingOnTimeOut)
						{
							gettimeofday(&tv_timeout, NULL);
							timeout_start_time = tv_timeout.tv_sec;
							WaitingOnTimeOut = 1;
						}
						else
						{
							gettimeofday(&tv_timeout, NULL);
							if (tv_timeout.tv_sec - timeout_start_time == CLIENT_TIMEOUT_VAL)
							{
								if (send_wrapper(MSG_ERROR_TIMEOUT, strlen(MSG_ERROR_TIMEOUT)) == -1)
								{
								 	/* if EPIPE, then peer disconnected */
                                        				if (errno == EPIPE)
                                                				goto END_SESSION;
                                        				else
                                               				{
										KMTLog_err("kmtd failed on send_wrapper()");
										syslog(LOG_ERR, "Existing on failure in send: %s\n", strerror(errno));
										exit(-1);	
									}
								}
								/* Goto outter loop */
								goto BEGIN_LOOP;
							}
						}
							
					}
				}
				else //bytes == N
				{
					char *datap = &data[strlen(MSG_UPDATE_CONF) + 2 /* spaces */ + N_len];
					datap[N] = '\0'; //cut off the \n

					/* When we write the config, we reload it as well, and then */
					/* execute/process the tools */


					if (ConfirmConfig(datap, EXECUTE_TOOLS) == PEER_DISCONNECT)
						goto END_SESSION;
				}
			}
			else
			if (strncasecmp(data, MSG_REQ_STATUS, 10) == 0)
			{
				sm = SendStats();
				if (sm == PEER_DISCONNECT || sm == SOCKET_ERR)
					goto END_SESSION;
			}
			else
			if (strncasecmp(data, MSG_FIND_SYSMAP, strlen(MSG_FIND_SYSMAP)) == 0)
			{
				if ((sm = SendSYSMAPS()) == PEER_DISCONNECT)
					goto END_SESSION;
				if (sm == SOCKET_ERR)
					goto END_SESSION;
			}	
			else
			if (strncasecmp(data, MSG_FIND_KIMAGE, strlen(MSG_FIND_KIMAGE)) == 0)
			{
				if ((sm = SendKimages()) == PEER_DISCONNECT)
                                        goto END_SESSION;
                                if (sm == SOCKET_ERR)
                                        goto END_SESSION;
			}
			else
			if (strncasecmp(data, MSG_EXEC_TOOL, strlen(MSG_EXEC_TOOL)) == 0)
			{
				p = &data[strlen(MSG_EXEC_TOOL) + 1];
				
				if (opts.debug)
					printf("[+] Received EXEC_TOOL: %s\n", p);
				
				memset((int *)tool_status, 0, sizeof(tool_status));

				if (strncasecmp(p, "kmemstat", 8) == 0)
				{
					if (tool_list.kmemstat)
						process_tool(&config.kmemstat, SINGLE_MODE);
					else
						SendMsg("-ERR UNCONFIGURED [Enable tool Kmemstat]\n");
				}
				else
				if (strncasecmp(p, "ksymstat", 8) == 0)
                                {
					if (tool_list.ksymstat)
				        	process_tool(&config.ksymstat, SINGLE_MODE);
                                	else
						SendMsg("-ERR UNCONFIGURED [Enable tool Ksymstat]\n");
				}
				else
				if (strncasecmp(p, "dps", 3) == 0)
                                {
					if (tool_list.dps)
				        	process_tool(&config.dps, SINGLE_MODE);
					else
						SendMsg("-ERR UNCONFIGURED [Enable tool DPS]\n");
				}
				else
				if (strncasecmp(p, "ktraq", 5) == 0)
                                {
					if (tool_list.ktraq)
			        		process_tool(&config.ktraq, SINGLE_MODE);
					else
						SendMsg("-ERR UNCONFIGURED [Enable tool Ktraq]\n");
				}
				else
				if (strncasecmp(p, "dr_chk", 6) == 0)
				{
					if (tool_list.dr_chk)
						process_tool(&config.dr_chk, SINGLE_MODE);
					else
						SendMsg("-ERR UNCONFIGURED [Enable tool dr_chk]\n");
				}

			}
			else
			if (strncasecmp(data, MSG_TAIL_LOG, strlen(MSG_TAIL_LOG)) == 0)
			{
				if (opts.debug)
					printf("[+] TAIL_LOG request\n");

				if (tail_in_progress == YES)
				{
					SendMsg("-ERR TAIL_IN_PROGRESS\n");
					continue;
				}
				
				/* NOTE We use temporary struct tail_args to pass data */
				/* to threaded function tail_dummy() and tail_logfile */
				char *tp = &data[strlen(MSG_TAIL_LOG + 1) + 1];
				while (*tp == 0x20)
					tp++;
				if (opts.debug)
					printf("tp: %s\n", tp);
				if (strncasecmp(tp, "*", 1) == 0)
					tail_args_data.logfile = strdup("/var/log/kmt/*.log");
				else
				if (strncasecmp(tp, "DPS", 3) == 0)
					tail_args_data.logfile = strdup("/var/log/kmt/dps.log");
				else
				if (strncasecmp(tp, "KMEMSTAT", 8) == 0)
					tail_args_data.logfile = strdup("/var/log/kmt/kmemstat.log");
				else
				if (strncasecmp(tp, "KSYMSTAT", 8) == 0)
					tail_args_data.logfile = strdup("/var/log/kmt/ksymstat.log");
				else
				if (strncasecmp(tp, "DR_CHK", 6) == 0)
					tail_args_data.logfile = strdup("/var/log/kmt/dr_chk.log");
				else
				if (strncasecmp(tp, "KTRAQ", 5) == 0)
					tail_args_data.logfile = strdup("/var/log/kmt/ktraq.log");
	
					
				/* Lets move to $LINES */
				while (*tp != 0x20)
					tp++;
				while (*tp == 0x20)	
					tp++;
				
				tail_args_data.lines = atoi(tp);
				
				if (opts.debug)
					printf("[+] TAIL_LOG: logfile (%s) lines (%d)\n", 
					 tail_args_data.logfile, tail_args_data.lines);
				
				/* Set an attribute for detached state */
				pthread_attr_setdetachstate(&detached_attr, PTHREAD_CREATE_DETACHED);
				
				pthread_create(&tail_thread, &detached_attr, 
				tail_dummy, (struct tail_args *)&tail_args_data);
				
				/* The tail -f has begun, if the client wishes it to stop */
				/* then it shall send over a command that says TAIL_LOG_END\n */
				tail_in_progress = YES;
			}
			else
			if (strncasecmp(data, MSG_TAIL_END, strlen(MSG_TAIL_END)) == 0)
			{	
				if (opts.debug)
					printf("[+] TAIL_END (tail_in_progress: %d)\n", tail_in_progress);
		
				if (tail_in_progress == NO)
				{
					SendMsg("-ERR TAIL_NOT_IN_PROGRESS\n");
					continue;
				}
				/* possibly add error checking to pthread_cancel */
				pthread_cancel(tail_thread);	
				/* Kill the actual /bin/tail processes */
				kill_tails(kmtd_pgid);
				tail_in_progress = NO;
				SendMsg("+TAIL_OK\n");
			}
		
	   }
	END_SESSION:
	if (tail_in_progress == YES)
	{
		pthread_cancel(tail_thread);
		kill_tails(kmtd_pgid);
		tail_in_progress = NO;
	}
	if (opts.debug)
		printf("[+] END_SESSION: GOTO WAIT_ACCEPT;\n");
	//TODO: close sfd
	PrevSessEnded = 1;
	ActiveSession = 0;
	goto WAIT_ACCEPT;
}
	
	
	


				
