/*
 *   KMTD (C) Innosecc.com 2009.
 *   The Kernel malware tracker Daemon. This software abides by the KMTD protocol
 *   as specified in KMTD-PROTOCOL file.
 *   Author of Daemon: Ryan@Innosecc.com
 *   Author of Client: Sean@Innosecc.com
 */      

#include "kmtd.h"

#define MAX_LINE 512
#define COMMENT_START "<!--"
#define COMMENT_END   "-->"

extern struct opts opts;

/* returns NULL on failure */
char * get_element_value(char *line, char *element)
{
	int i = 0, j, slen = strlen(element);
	char *p = line;
	char *s, *val;

	if ((val = (char *)malloc(255)) == NULL)
	{
		KMTLog_err("malloc() failed: %s\n", strerror(errno));
		exit(-1);
	}

	while (i < MAX_LINE)
	{
		if (strncasecmp(&p[i], element, slen) == 0)
		{
			s = &p[i + slen];
			s += 2; 
			for (j = 0; *s != '"'; j++, s++)
				val[j] = *s;
			return val;
		}
		i++;
	}
	if (opts.debug)
		printf("XML Element <%s> does not appear to exist on line: %s\n", element, line);
	return (char *)0;
}			


int Read_KMD_Config(char *path, struct config_info *config)
{
	FILE *fd;
	char line[512], element[255] = {0};
	char tmp_buf[255]; 
	char *s, *p;
	char in_stanza = 0, in_comment = 0;
	int i, j, current_tool;
	int val;
	extern struct tool_list tool_list;
	extern char global_zip_passwd[32];
	extern int ACTIVE_TOOL_COUNT;
	extern char *email[255];
	extern int email_count;
	extern int SendAlerts;
	extern char mailutil[255];

	/* Open config file */
	if ((fd = fopen(path, "r")) == NULL)
	{
		snprintf(tmp_buf, sizeof(tmp_buf), "Cannot read config: %s", path);
		perror(tmp_buf);
		return -1;
	}
	
	next:
	while (fgets(line, sizeof(line), fd))
	{
		s = line;
		while (*s == 0x20)
			s++;	
		
		/* Handle comments (nested comments not supported */
		
		if ((strncmp(s, COMMENT_START, strlen(COMMENT_START)) == 0) && strstr(s, COMMENT_END))
			continue;

		if (in_comment == 0)
		{
			if (strncmp(s, COMMENT_START, strlen(COMMENT_START)) == 0)
			{
				in_comment = 1;
				continue;
			}
		}
		
		if (in_comment == 1)
		{
			if (strncmp(s, COMMENT_END, strlen(COMMENT_END)) == 0)
			{	
				in_comment = 0;
				continue;
			}
			else
				continue;
		}
	
		/* End of comment handling */
			
		
		if (*s != '<')
			continue;
		s++; 
		memset(element, 0, 255);
		for (i = 0; *s != '>'; i++, s++)
			element[i] = *s;
		element[i] = '\0';

		if (strcasecmp(element, "tool") == 0)
		{
			next_line:
			fgets(line, sizeof(line), fd);
			if ((s = strchr(line, '<')) == NULL)
				goto next_line;
			s++;
			if (strncasecmp(s, "exec_info", 9) == 0)
			{
				if (strcasecmp(get_element_value(s, "name"), "kmemstat") == 0)
				{
					tool_list.kmemstat = 1;
					config->kmemstat.toolID = current_tool = TOOL_KMEMSTAT;
					ACTIVE_TOOL_COUNT++;
					if ((config->kmemstat.tool_name = strdup(get_element_value(s, "name"))) == NULL)
						return -1;
					if ((config->kmemstat.exec_path = strdup(get_element_value(s, "path"))) == NULL)
						return -1;
					if ((config->kmemstat.option_str = strdup(get_element_value(s, "opts"))) == NULL)
						return -1;
				}
				else
				if (strcasecmp(get_element_value(s, "name"), "ksymstat") == 0)
                                {
					tool_list.ksymstat = 1;
					config->ksymstat.toolID = current_tool = TOOL_KSYMSTAT;
					ACTIVE_TOOL_COUNT++;
                                        if ((config->ksymstat.tool_name = strdup(get_element_value(s, "name"))) == NULL)
                                                return -1;
                                        if ((config->ksymstat.exec_path = strdup(get_element_value(s, "path"))) == NULL)
                                                return -1;
                                        if ((config->ksymstat.option_str = strdup(get_element_value(s, "opts"))) == NULL)
                                                return -1;
                                }
				else
				if (strcasecmp(get_element_value(s, "name"), "ktraq") == 0)
                                {
					tool_list.ktraq = 1;
					config->ktraq.toolID = current_tool = TOOL_KTRAQ;
					ACTIVE_TOOL_COUNT++;
                                        if ((config->ktraq.tool_name = strdup(get_element_value(s, "name"))) == NULL)
                                                return -1;
                                        if ((config->ktraq.exec_path = strdup(get_element_value(s, "path"))) == NULL)
                                                return -1; 
                                        if ((config->ktraq.option_str = strdup(get_element_value(s, "opts"))) == NULL)
                                                return -1;
                                }
				else
				if (strcasecmp(get_element_value(s, "name"), "dps") == 0)
                                {
					tool_list.dps = 1;
					config->dps.toolID = current_tool = TOOL_DPS;
					ACTIVE_TOOL_COUNT++;
                                        if ((config->dps.tool_name = strdup(get_element_value(s, "name"))) == NULL)
                                                return -1;
                                        if ((config->dps.exec_path = strdup(get_element_value(s, "path"))) == NULL)
                                                return -1;
                                        if ((config->dps.option_str = strdup(get_element_value(s, "opts"))) == NULL)
                                                return -1;
                                }
				else
				if (strcasecmp(get_element_value(s, "name"), "dr_chk") == 0)
                                {
                                        tool_list.dr_chk = 1;
                                        config->dr_chk.toolID = current_tool = TOOL_DR_CHK;
                                        ACTIVE_TOOL_COUNT++;
                                        if ((config->dr_chk.tool_name = strdup(get_element_value(s, "name"))) == NULL)
                                                return -1;
                                        if ((config->dr_chk.exec_path = strdup(get_element_value(s, "path"))) == NULL)
                                                return -1;
                                        if ((config->dr_chk.option_str = strdup(get_element_value(s, "opts"))) == NULL)
                                                return -1;
                                }

				goto next_line;

			}
			else
			if (strncasecmp(s, "os_support", 10) == 0)
			{

				if (current_tool == TOOL_KMEMSTAT)
				{
					if (!strcasecmp(get_element_value(s, "linux"), "yes"))
						config->kmemstat.supported_by_linux = 1;
                                        else
						config->kmemstat.supported_by_linux = 0;
					
					if (!strcasecmp(get_element_value(s, "freebsd"), "yes"))
                                                config->kmemstat.supported_by_freebsd = 1;
                                        else
                                                config->kmemstat.supported_by_freebsd = 0;

				}
				else
				if (current_tool == TOOL_KSYMSTAT)
				{
					if (!strcasecmp(get_element_value(s, "linux"), "yes"))
						config->ksymstat.supported_by_linux = 1;
					else
						config->ksymstat.supported_by_linux = 0;
					
					if (!strcasecmp(get_element_value(s, "freebsd"), "yes"))
						config->ksymstat.supported_by_freebsd = 1;
					else
						config->ksymstat.supported_by_freebsd = 0;

				}
				else
				if (current_tool == TOOL_KTRAQ)
                                { 
					if (!strcasecmp(get_element_value(s, "linux"), "yes"))
                                                config->ktraq.supported_by_linux = 1;
                                        else
                                                config->ktraq.supported_by_linux = 0;

                                        if (!strcasecmp(get_element_value(s, "freebsd"), "yes"))
                                                config->ktraq.supported_by_freebsd = 1;
                                        else
                                                config->ktraq.supported_by_freebsd = 0;
      
                                }
                                else
                                if (current_tool == TOOL_DPS)
                                { 
					if (!strcasecmp(get_element_value(s, "linux"), "yes"))
                                                config->dps.supported_by_linux = 1;
                                        else
                                                config->dps.supported_by_linux = 0;

                                        if (!strcasecmp(get_element_value(s, "freebsd"), "yes"))
                                                config->dps.supported_by_freebsd = 1;
                                        else
                                                config->dps.supported_by_freebsd = 0;

                                }
	                        else
                                if (current_tool == TOOL_DR_CHK)
                                {
                                        if (!strcasecmp(get_element_value(s, "linux"), "yes"))
                                                config->dr_chk.supported_by_linux = 1;
                                        else
                                                config->dr_chk.supported_by_linux = 0;

                                        if (!strcasecmp(get_element_value(s, "freebsd"), "yes"))
                                                config->dr_chk.supported_by_freebsd = 1;
                                        else
                                                config->dr_chk.supported_by_freebsd = 0;

                                }

				goto next_line;
			}
			else
			if (strncasecmp(s, "logfile", 7) == 0)
			{

				char *p2, logfile[255];
				

				/* BIG TODO */
				/* If the config file is re-read various times, each time */
				/* it allocates memory, over time this would become too much */
				/* allocated memory. We need to see if the logfile ptr has been */
				/* allocated already (logfile_name) */
				
				/* pre-allocated logfile_name's in data segment */
				if (current_tool == TOOL_KMEMSTAT)
					p2 = config->kmemstat.logfile_name; // = malloc(255);
				else
				if (current_tool == TOOL_KSYMSTAT)
                                        p2 = config->ksymstat.logfile_name;// = malloc(255);
 				else
				if (current_tool == TOOL_KTRAQ)
                                        p2 = config->ktraq.logfile_name; //= malloc(255);
				else
				if (current_tool == TOOL_DPS)
					p2 = config->dps.logfile_name; //= malloc(255);
				else
				if (current_tool == TOOL_DR_CHK)
					p2 = config->dr_chk.logfile_name; //= malloc(255);
				
				memset(p2, 0, 255);
				p = strchr(s, '>');
				p++;
				for (i = 0; *p != '<'; i++, p++)
					logfile[i] = *p;
				logfile[i] = '\0';
				strcpy(p2, logfile);

				goto next_line;
			}
			else
			if (strncasecmp(s, "enabled", 7) == 0)
			{
				int enabled_val;
			    	char *toolname = "";

    				p = s;
                        	while (*p != '>')
                                	p++;
                        	p++;
                        	for (j = 0; *p != '<'; j++, p++)
                                	tmp_buf[j] = *p;
                        	tmp_buf[j] = '\0';
				/* even if it is set to 2, it will be 1 in enabled_val*/
				enabled_val = atoi(tmp_buf) == 0 ? 0 : 1;
				switch(current_tool)
				{
					case TOOL_KMEMSTAT:
						toolname = "kmemstat";
						config->kmemstat.enabled = enabled_val;
						break;
					case TOOL_KSYMSTAT:
						toolname = "ksymstat";
						config->ksymstat.enabled = enabled_val;
						break;
					case TOOL_KTRAQ:
						toolname = "ktraq";
						config->ktraq.enabled = enabled_val;
						break;
					case TOOL_DPS:
						toolname = "dps";
						config->dps.enabled = enabled_val;
						break;
					case TOOL_DR_CHK:
						toolname = "dr_chk";
						config->dr_chk.enabled = enabled_val;
						break;
				}

				if (opts.debug)
					printf("[+] Tool <%s> is %senabled\n", toolname, enabled_val == 0 ? "not " : "");
			} /* end enabled */
		}
		else
		if (strncasecmp(element, "time_interval", 13) == 0)
		{
			p = s;
			while (*p != '>')
				p++;
			p++;
			for (val = 0; *p != '<'; p++)
				val = 10 * val + (*p - '0');
			config->time_interval = val;
			if (opts.debug)
				printf("[+] config->time_interval: %d\n", config->time_interval);
		}
		else
		if (strncasecmp(element, "kernel_image", 12) == 0)
		{
			p = s;
			while (*p != '>')
				p++;
			p++;
			for (j = 0; *p != '<'; j++, p++)
				tmp_buf[j] = *p;
			tmp_buf[j] = '\0';
			config->kernel_image = strdup(tmp_buf);
			if (config->kernel_image == NULL)
				return -1;
		}
		else
                if (strncasecmp(element, "system_map", 10) == 0)
                {
                        p = s;
                        while (*p != '>')
                                p++;
                        p++;
                        for (j = 0; *p != '<'; j++, p++)
                                tmp_buf[j] = *p;
                        tmp_buf[j] = '\0';
                        config->system_map = strdup(tmp_buf);
                        if (config->system_map == NULL)
                                return -1;
                }
		else 
		if (strncasecmp(element, "email_list", 10) == 0)
		{
			p = s;
			while (*p != '>')
				p++;
			p++;

			email_loop:
			for (j = 0; *p != '<' && *p != ','; j++, p++)
                                tmp_buf[j] = *p;

			tmp_buf[j] = '\0';
			email[email_count++] = (char *)strdup(tmp_buf);
			if (*(p) == ',')
			{
				p++;
				goto email_loop;
			}
		}
		else 
		if (strncasecmp(element, "send_alerts", 11) == 0)
                {
                        p = s;
                        while (*p != '>')
                                p++;
                        p++;
                        for (j = 0; *p != '<'; j++, p++)
                                tmp_buf[j] = *p;
                        tmp_buf[j] = '\0';
			SendAlerts = atoi(tmp_buf);
			if (opts.debug)
				printf("[+] SendAlerts: %d\n", SendAlerts);
		}
		else
		if (strncasecmp(element, "zip_passwd", 10) == 0)
		{
			                 p = s;
                        while (*p != '>')
                                p++;
                        p++;
                        for (j = 0; *p != '<'; j++, p++)
                                tmp_buf[j] = *p;
                        tmp_buf[j] = '\0';
			strncpy(global_zip_passwd, tmp_buf, sizeof(global_zip_passwd));
			global_zip_passwd[sizeof(global_zip_passwd) - 1] = '\0';
			if (opts.debug)
				printf("[+] ZIP Passwd: %s\n", global_zip_passwd);
		}
		else
		if (strncasecmp(element, "jkmt_passwd", 11) == 0)
		{
			                 p = s;
                        while (*p != '>')
                                p++;
                        p++;
                        for (j = 0; *p != '<'; j++, p++)
                                tmp_buf[j] = *p;
                        tmp_buf[j] = '\0';
			config->jkmt_passwd = strdup(tmp_buf);
		    /* no printing this out to stdout */
		}
		else
		if (strncasecmp(element, "mailutil", 8) == 0)
                {
                                         p = s;
                        while (*p != '>')
                                p++;
                        p++;
                        for (j = 0; *p != '<'; j++, p++)
                                tmp_buf[j] = *p;
                        tmp_buf[j] = '\0';
			strncpy(mailutil, tmp_buf, sizeof(mailutil));
			mailutil[sizeof(mailutil) - 1] = '\0';
			if (opts.debug)
				printf("[+] mailutil: %s\n", mailutil);
		}
		else
		/* ReadConfig was returning -1 after getting time_interval */
		/* after the config.c changes you made, adding this worked */
		/* and should have been present anyway */
		if (strncasecmp(element, "</root_element>", 15) == 0)
		{
			return 0;
		}
		
	}
	fclose(fd);
}

int WriteConfig(char *path, char *data)
{
	char old_config[256];
	FILE *fd;

	/* back up the original */

	strncpy(old_config, path, 250);
	strncat(old_config, ".bak", 4);
	rename(path, old_config);
	
	if ((fd = fopen(path, "w+")) == NULL)
	{
		perror("WriteConfig failed -> fopen");
		return -1;
	}
	
	fprintf(fd, "%s", data);
	fflush(fd);
	fclose(fd);
}

	
