#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/types.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <getopt.h>
#include <string.h>
#include <errno.h>
#include <locale.h>
#include <elf.h>
#include <sys/mman.h>

#include <netinet/in.h>
#include <netdb.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <time.h>
#include <pthread.h>

#include <signal.h>
#include <syslog.h>
#include <setjmp.h>
#include <sys/utsname.h>
#include <sys/wait.h>

#include <openssl/md5.h>
#include <openssl/ssl.h>
#include <openssl/err.h>

#ifdef __FreeBSD__
#include <kvm.h>
#include <nlist.h>
#endif

#define NO 0
#define YES 1

#define MSG_TAIL_LOG "TAIL_LOG"
#define MSG_TAIL_END "TAIL_END"
#define MSG_ERROR_TIMEOUT "-ERR TIMEOUT"
#define MSG_ERROR_LEN_MISMATCH "-ERR LEN_MISMATCH"
#define MSG_OK "+OK"
#define MSG_UPDATE_CONF "UPDATE_CONF"
#define MSG_ALERT "ALERT"
#define MSG_REQ_STATUS "REQ_STATUS"
#define MSG_FIND_SYSMAP "FIND_SYSMAP"
#define MSG_FIND_KIMAGE "FIND_KIMAGE"
#define MSG_EXEC_TOOL "EXEC_TOOL"
#define CLIENT_TIMEOUT_VAL 5
/* Tools send daemon this string if malware is found */
#define ALERT_DAEMON "[ALERT_TO_DAEMON]"

#define PAUSE_A_MOMENT usleep(100)

#define TOOL_T_FLAG -1

#define TOOL_KMEMSTAT 0
#define TOOL_KSYMSTAT 1
#define TOOL_KTRAQ 2
#define TOOL_DPS 3
#define TOOL_DR_CHK 4

#define KMTD_PORT 999

#define PEER_DISCONNECT -2
#define SOCKET_ERR -3

#define DEFAULT_LOG_PATH "/var/log/kmt/kmtd.log"
#define DEFAULT_CONF_PATH "/opt/kmt/kmt_config/kmt.conf"

#define ALERT_GUI 1

#define EXECUTE_TOOLS 1
#define NO_EXECUTE 0

#define ERR_FP_SYSMAP -1
#define ERR_FP_KERNIMG -2
#define ERR_SYSMAP_MISMATCH 1
#define ERR_KERNIMG_MISMATCH 2

#define SYSMAP_PANIC 0
#define KERNIMG_PANIC 1
#define MISC_PANIC 2

/* For process_tool() */
#define MULTI_MODE 0
#define SINGLE_MODE 1

struct tool_list
{
	char kmemstat, ksymstat, ktraq, dps, dr_chk;
};

typedef struct rk_tool_options
{
	int toolID;
	int enabled; /* 0 disabled */
	char *tool_name;
	char *exec_path;
	char logfile_name[255]; // within /var/log/kmd
	char *option_str; // i.e "-k vmlinux -a"
	char supported_by_linux;
	char supported_by_freebsd;
} rk_tool_opts;

struct config_info
{
	rk_tool_opts kmemstat;
	rk_tool_opts ktraq;
	rk_tool_opts dps;
	rk_tool_opts ksymstat;
	rk_tool_opts dr_chk;
	unsigned int time_interval;
	char *system_map;
	char *kernel_image;
	char *jkmt_passwd; /* TODO: rename to jkmt_passwd_hash once we are using SHA1 or -lcrypt passwd hash */
};

struct opts
{
        char debug;
        char verbose;
        char custom_log_path;
        char custom_conf_path;
        char *logpath;
        char *config;
};
#ifdef x86_64 
        typedef Elf64_Ehdr ElfX_Ehdr;
        typedef Elf64_Phdr ElfX_Phdr;
        typedef Elf64_Shdr ElfX_Shdr;
        typedef Elf64_Addr ElfX_Addr;
        typedef Elf64_Off  ElfX_Off;
        typedef Elf64_Word ElfX_Word;
#else
        typedef Elf32_Ehdr ElfX_Ehdr;
        typedef Elf32_Phdr ElfX_Phdr;
        typedef Elf32_Shdr ElfX_Shdr;
        typedef Elf32_Addr ElfX_Addr;
        typedef Elf32_Off  ElfX_Off;
        typedef Elf32_Word ElfX_Word;

#endif

#define ZIPDIR "/var/log/kmt/.zip/"
#define ZIP "/usr/bin/zip -P "
#define AUTH "abc" //passwd to unzip logs

#define CLEAN_EXIT 1
#define CRASH_EXIT 2
#define SIGNL_EXIT 4
#define CORED_EXIT 8

/* To pass args through a pthread for tail_logfile() */
struct tail_args
{
	int sfd;
	int lines;
	char *logfile;
};

int locate_system_map(char *);
int VerifySigFP(char *, char *);
void sighandle(int);
void NotifyUsers(char *, ...);
void Write2Log (char *);
void KMTLog(char *, char *);
int Read_KMD_Config(char *path, struct config_info *config);
int WriteConfig(char *path, char *data);
int bind_connection(void);
int FPSigs(char *, char *);
int GetMD5 (char *, char *);
char *getMD5_2(char *);
int tail_logfile(int sfd, char *, int);
void * tail_dummy(void *);
char ** get_sysmap_paths(int *count, int *len);
