#include "kmtd.h"

static SSL_CTX *the_ctx;

static int
ssld_accept (int cipher_sock, SSL ** ret)
{
	int s = cipher_sock;

	BIO *sbio = BIO_new_socket (s, BIO_NOCLOSE);
	SSL *ssl = SSL_new (the_ctx);
	SSL_set_bio (ssl, sbio, sbio);

	int r;
	if ((r = SSL_accept (ssl)) <= 0)
		return -1;

	*ret = ssl;
	return 0;
}

static int
ssld_send (SSL * ssl, void *buf, size_t count, int flags)
{
	int r = SSL_write (ssl, (char *) buf, count);
	if (r <= 0)
	{
		openssl_print_error (ssl, r, 1);
		return -1;
	}
	return r;

}

static int
ssld_recv (SSL * ssl, void *buf, size_t count, int flags)
{
	int r = SSL_read (ssl, (char *) buf, count);
	if (r <= 0)
	{
		openssl_print_error (ssl, r, 1);
		return -1;
	}
	return r;
}


static int
ssld_close (SSL * ssl)
{
	SSL_shutdown (ssl);
	SSL_free (ssl);
	return 0;
}

static SSL_CTX *
ssl_init (const char *server_pem, const char *dh_pem, const char *calist_pem)
{
	SSL_CTX *ctx;
	SSL_library_init ();
	SSL_load_error_strings ();

	SSL_METHOD *meth = SSLv23_method ();
	ctx = SSL_CTX_new (meth);

	if (!(SSL_CTX_use_certificate_chain_file (ctx, server_pem)))
		printf ("Can't read certificate file %s", server_pem);
	if (calist_pem)
		if (!(SSL_CTX_load_verify_locations (ctx, calist_pem, 0)))
			printf ("Can't read CA list %s", calist_pem);
	if (OPENSSL_VERSION_NUMBER < 0x00905100L)
		SSL_CTX_set_verify_depth (ctx, 1);
	if (dh_pem)
	{
		DH *ret = 0;
		BIO *bio;

		if (!(bio = BIO_new_file (dh_pem, "r")))
			printf ("Couldn't open DH file %s", dh_pem);
		ret = PEM_read_bio_DHparams (bio, NULL, NULL, NULL);
		BIO_free (bio);
		if (SSL_CTX_set_tmp_dh (ctx, ret) < 0)
			printf ("Couldn't set DH parameters using %s", dh_pem);
	}

	return ctx;
}
