/*
 * Query.c - Ktraq IDS - Copyright 2008 Ryan O'Neill <ryan@bitlackeys.com> 
 *
 * This code generates a temporary file called syscalls.map which reflects where certain kernel
 * symbols reside in memory, and then queries their authenticity.
 * 
 */


#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <unistd.h>
#include <locale.h>
#include <stdlib.h>
#include <time.h>

#define MAX_SYSCALL 256

#define SYSTAB "syscall.tab"
#define SCA "/tmp/sca.tmp"
#define SYSCALLS "syscalls.map"	/* basically our own system.map of syscalls only, that reflects current memory */
#define SYSTEM_MAP "/boot/System.map"
#define SYSFIX "sys.fix"

const char *sys_array[] = {
#define S_NUM 12
"sys_chdir",
"sys_open",
"sys_write",
"sys_getdents",
"sys_getdents64",
"sys_setuid",
"sys_setuid32",
"sys_ioctl",
"sys_fork",
"sys_clone",
"sys_kill",
"sys_unlink",
NULL
};

struct sys_call_s
{
   char syscall[64];
   unsigned long address;
}  sys_call_entry[MAX_SYSCALL];

struct sys_map_s
{
   char syscall[64];
   char c;
   unsigned long address;
}  sys_map_entry[26000];

struct sys_restore 
{
 //  char syscall[64];
   int number;
   unsigned long address; /* correct address */
} sys_restore_entry[32];

int
main (int argc, char **argv)
{
  int i = 0, count = 0, j = 0, k = 0, z = 0, rc = 0;
  int sz, len;
 
  time_t clock;
  struct tm *tm;
      
  FILE *fd, *sd, *sm, *sp, *fx;
  
  char *p, tmp[64], smp[64];
  char timebuf[96];
  
  char intercepted[350][250]; 


  if (argc < 2)
  {
  	printf("This internal tool is part of the ktraq IDS\n"); 
	printf("Usage: %s <sys.map>\n",argv[0]);
	exit (-1);
  }

  if ((sp = fopen (argv[1], "r")) == NULL)
  {
      perror ("fopen");
      exit (-1);
  }

  if ((sm = fopen (SYSCALLS, "w")) == NULL)
  {
      perror ("fopen: syscalls.map");
      exit (0);
  }

  if ((fd = fopen (SYSTAB, "r")) == NULL)
  {
      perror ("fopen: syscall.tab");
      exit (-1);
  }

  if ((sd = fopen (SCA, "r")) == NULL)
  {
      perror ("fopen: /tmp/sca.tmp");
      exit (-1);
  }

  
  /* 
   * this merges the syscall names with the syscall numbers
   */

  while (fgets (tmp, 64, fd) != NULL && fgets (smp, 64, sd) != NULL)
  {
     	 if (tmp[strlen (tmp) - 1] == '\n')
	     tmp[strlen (tmp) - 1] = '\0';

      	sprintf (sys_call_entry[i].syscall, "sys_%s", tmp);
      	sys_call_entry[i].address = strtoul (smp, NULL, 16);
        i++;
  }

  /* 
   * we write our syscalls.map file here, which is basically
   * a new system.map file, that contains symbols from the sys_call_table only
   */
  for (i = 0; i < MAX_SYSCALL; i++)
  {
        fprintf (sm, "%x %s\n", sys_call_entry[i].address,
	       sys_call_entry[i].syscall);
        fflush (sm);
  }

  /*
   * Here we read the symbols from the original system.map file
   * into an array of a structure pointed to by sys_map_entry
   */
  count = 0;
  for (i = 0; !feof (sp); i++)
  {
        fscanf (sp, "%x %c %s", &sys_map_entry[i].address, &sys_map_entry[i].c,
	      &sys_map_entry[i].syscall);
        count++;
  }

 
  j = 0;
  for (i = 0; i < MAX_SYSCALL; i++)
  {      
       for (j = 0; j < count; j++)
       {
    
           len = strlen(sys_call_entry[i].syscall);
 
               for (k = 0; k < S_NUM; k++) 
               if (strcmp(sys_map_entry[j].syscall, sys_array[k]) == 0)
               {	
	           if (strncmp(sys_call_entry[i].syscall, sys_map_entry[j].syscall, len+1) == 0)
	   	   { 
			if ((u_long)sys_call_entry[i].address != (u_long)sys_map_entry[j].address)
			{
	                	sprintf(intercepted[z], "%s was found at %x, it should be at %x\n",
				sys_call_entry[i].syscall, sys_call_entry[i].address, sys_map_entry[j].address); 
				z++; 
	                        sys_restore_entry[rc].number = i;
				sys_restore_entry[rc].address = sys_map_entry[j].address;
				rc++;
		        } 
			break;
	            } 
		}

         }
	continue;
  }

  
  if (z > 0)
  { 
       
        time (&clock);
        tm = gmtime ((time_t *)&clock);
        setlocale (LC_TIME, "C");
        strftime (timebuf, sizeof (timebuf), "%A, %B %e, %Y, %H:%M:%S", tm);
    
        printf("On %s Ktraq v0.2 discovered a malicious rootkit that has intercepted [%d] systemcalls.\n",timebuf,z);
	printf("Here are the details:\n");
 
        for (i = 0; i < z; i++)
		printf("%s",intercepted[i]);

	if ((fx = fopen(SYSFIX, "w")) == NULL)
		perror("fopen: sys.fix");
	
        for (i = 0; i < rc; i++)
	{
		fprintf(fx, "%d:0x%x:", 
		sys_restore_entry[i].number, 
		sys_restore_entry[i].address);
	} 

	fclose(fx);

	printf("Ktraq IDS, by Ryan O'Neill <ryan@bitlackeys.com>\n");
  }

else   {
		printf("Ktraq v0.2 - has found your OS's system call table to be clean...\n");
		printf("This means that no LKM, or /dev/kmem style rootkit is installed!\n"); 
        	printf("Ktraq IDS, By Ryan O'Neill <ryan@bitlackeys.com>\n");
       } 


  fclose (fd);
  fclose (sd);
  fclose (sm);
  fclose (sp);

}             
