/*
 * Totally rewritten Ktraq code
 * Ktraq v2.0 (C) Innosecc.com 2009
 * Author: Ryan O'Neill
 */


#include "ktraq.h"
#include <errno.h>
#include <sys/stat.h>

#define INSMOD "/sbin/insmod"
#define MODULE "/opt/kmt/sbin/ktraq_module/ktraq.ko"
#define RMMOD "/sbin/rmmod"

//#define SYSCALLS "/opt/kmt/ktraq_data/syscalls.map"
#define SYSTAB "/opt/kmt/ktraq_data/syscall.tab"
#define SCA "/tmp/sca.tmp"

#define MAX_SYSCALL 256

/* This struct gets data from sys.map */
struct sys_call_s
{
   char syscall[64];
   unsigned long address;
}  sys_call_entry[MAX_SYSCALL];

struct kallsyms
{
   char syscall[64];
   char c;
   unsigned long address;
}  kallsyms_entry[26000];


void
exec_string (char *str, ...)
{
        char string[255];
        va_list va;

        va_start (va, str);
        vsnprintf (string, 255, str, va);
        va_end (va);

        system (string);
}

int main(int argc, char **argv)
{ 
	int c, i, j, count = 0, k;
	extern char *optarg;
	unsigned long sys_call_table;
	int use_kmt_mem = 0, use_sysmap = 0;
	FILE *fd, *pd, *sm, *sd, *kp;

	char tmp[64], smp[64], sct_vaddr[32];
	char *sysmap, line[255];
	struct stat st;

	while ((c = getopt(argc, argv, "hMs:")) != -1)
	{
		switch(c)
		{
		case 'M':
			use_kmt_mem++;
			break;
		case 's':
			if ((sysmap = strdup(optarg)) == NULL)
			{
				printf("[TOOL_FAILURE] strdup %s\n", strerror(errno));
				exit(-1);
			}
			use_sysmap++;
			break;
		case 'h':
			printf("\nUsage: %s [-hM] [-s <System.map> (Optional)]\n"
			       "[-M]	Use /dev/kmt_mem instead of /dev/kmem\n"
			       "[-s]    In addition to default behavior, verify\n"
			       "        correct sys_call_table by comparing it against System.map\n"
			       "[-r]	Restore syscall table if hijack detection results in rootkit tampering\n"
		 	       "[-h]	Help\n\n", argv[0]);
			exit(0);
			
				
			
		}
	}

	sys_call_table = get_sct(use_kmt_mem);
	if (sys_call_table == 0)
	{
		printf("[+] Unable to locate sys_call_table\n");
		exit(-1);
	}

	if (use_sysmap)
	{
		if (sys_call_table != get_symbol("sys_call_table", sysmap))
		{
			printf("[+] sys_call_table found in system_call() does not match the one in System.map\n"
			       "Run the tool 'dps' for a further analysis, since this could be an indication of a rootkit\n");
			exit(-1);
		}
	}
	
	if (stat(MODULE, &st) < 0)
	{
		printf("[TOOL_FAILURE] stat ktraq_module: %s %s\n", MODULE, strerror(errno));
		exit(-1);
	}
	
	sprintf(sct_vaddr, "0x%x", sys_call_table);
	/* Build file that contains symbol addresses reflecting current memory */
	exec_string ("%s %s sct=%s", INSMOD, MODULE, sct_vaddr);
	usleep(500);

	exec_string ("%s ktraq", RMMOD);

	/* syscall.tab file */
  	if ((fd = fopen (SYSTAB, "r")) == NULL)
  	{
      		printf("[TOOL_FAILURE] fopen: syscall.tab %s\n", strerror(errno));
      		exit (-1);
  	}

	/* /tmp/sca.tmp file */
 	if ((sd = fopen (SCA, "r")) == NULL)
  	{
		printf("[TOOL_FAILURE] fopen: /tmp/sca.tmp - %s\n", strerror(errno));
      		exit (-1);
  	}

	/* /proc/kallsyms */
	if ((kp = fopen("/proc/kallsyms", "r")) == NULL)
	{
		printf("[TOOL_FAILURE] fopen: /proc/kallsyms %s\n", strerror(errno));
		exit(-1);
	}

  	/*------------------------------------------------------------------- */
 	//	this merges the syscall names with the syscall addresses 
	i = 0;
	int syscount = 0;
  	while (fgets(tmp, 64,/* syscall.tab*/ fd) != NULL && i < 256)
	{
		fgets(smp, 64, /* /tmp/sca.tmp */ sd);
		if (tmp[strlen (tmp) - 1] == '\n')
             		tmp[strlen (tmp) - 1] = '\0';
		
		sprintf (sys_call_entry[i].syscall, "sys_%s", tmp);
        	sys_call_entry[i].address = strtoul (smp, NULL, 16);
        	i++;
  	}
	 // At this point, sys_call_entry array is equiv to a System.map that contains only
	 // syscalls, and we can compare it to /proc/kallsyms to see if the syscall
	 // pointers have been modified
 	 /*------------------------------------------------------------------- */
	
 	/*
	 *  Here we read the symbols from /proc/kallsyms
 	 *  into kallsyms_entry array
         */
  	char *p;
	i = 0;
	while (fgets(line, sizeof(line), kp))
  	{
		p = &line[11];
		if (strncmp(p, "sys_", 4) != 0)
			continue;
		p = &line[9];
		if (*p != 'T')
			continue;
        	sscanf (line, "%x %c %s", &kallsyms_entry[i].address, &kallsyms_entry[i].c,
              		kallsyms_entry[i].syscall);
		if (strcmp(kallsyms_entry[i].syscall, "") == 0)
			break;
		count++;
		i++;
  	}
	int found_similar, hijacked = 0;
	
	printf("\n[+] Scanning for modified syscall table entries...\n");
	for (i = 0; i < 255; i++)
       	{
		found_similar = 0;
		for (j = 0; j < count; j++)
			if (strcasecmp(kallsyms_entry[j].syscall, sys_call_entry[i].syscall) == 0)
			{	
				if (sys_call_entry[i].address != kallsyms_entry[j].address)
				{
					/* 
 					  It's possible that a syscall like sys_stat shows up as hijacked
					  because it points to sys_newstat. We need to be aware of this.
					  if and only if it points to outside of allocated memory should we
					  be suspicious 
					*/

					if (sys_call_entry[i].address < 0xc0000000 || sys_call_entry[i].address > 0xcfffffff)
					{
						hijacked++;
						if (hijacked == 1)
							printf("[ALERT_TO_DAEMON]\n");
						printf("[!] syscall %s has been redirected to 0x%x, but should be (%s) 0x%x\n", 
						sys_call_entry[i].syscall, sys_call_entry[i].address, kallsyms_entry[j].syscall, kallsyms_entry[j].address);
					}
				}
			}
	}

	if (!hijacked)
	{
		printf("[+] Congratulations! Ktraq has determined that no kernel rootkits which directly modify the syscall table are present\n");
		printf("[+] Run dps, ksymstat, dr_chk, or kmemstat to scan for any other types of kernel malware\n");
	}
	else
	{
		printf("\n[+] The sys_call_table has %d entries that have been modified\n", hijacked);	
		printf("\n[+] This means that a kernel rootkit is present!\n");
	}
	printf("\n");
	exit(0);
	

}
